@@ -248,17 +248,17 @@ extern void drbd_assert_breakpoint(struct drbd_conf *, char *, char *, int);
 
 /* Defines to control fault insertion */
 enum {
-    DRBD_FAULT_MD_WR = 0,	/* meta data write */
-    DRBD_FAULT_MD_RD = 1,	/*           read  */
-    DRBD_FAULT_RS_WR = 2,	/* resync          */
-    DRBD_FAULT_RS_RD = 3,
-    DRBD_FAULT_DT_WR = 4,	/* data            */
-    DRBD_FAULT_DT_RD = 5,
-    DRBD_FAULT_DT_RA = 6,	/* data read ahead */
-    DRBD_FAULT_BM_ALLOC = 7,	/* bitmap allocation */
-    DRBD_FAULT_AL_EE = 8,	/* alloc ee */
+	DRBD_FAULT_MD_WR = 0,	/* meta data write */
+	DRBD_FAULT_MD_RD = 1,	/*           read  */
+	DRBD_FAULT_RS_WR = 2,	/* resync          */
+	DRBD_FAULT_RS_RD = 3,
+	DRBD_FAULT_DT_WR = 4,	/* data            */
+	DRBD_FAULT_DT_RD = 5,
+	DRBD_FAULT_DT_RA = 6,	/* data read ahead */
+	DRBD_FAULT_BM_ALLOC = 7,	/* bitmap allocation */
+	DRBD_FAULT_AL_EE = 8,	/* alloc ee */
 
-    DRBD_FAULT_MAX,
+	DRBD_FAULT_MAX,
 };
 
 extern void trace_drbd_resync(struct drbd_conf *mdev, int level, const char *fmt, ...);
@@ -268,9 +268,9 @@ extern unsigned int
 _drbd_insert_fault(struct drbd_conf *mdev, unsigned int type);
 static inline int
 drbd_insert_fault(struct drbd_conf *mdev, unsigned int type) {
-    return fault_rate &&
-	    (enable_faults & (1<<type)) &&
-	    _drbd_insert_fault(mdev, type);
+	return fault_rate &&
+		(enable_faults & (1<<type)) &&
+		_drbd_insert_fault(mdev, type);
 }
 #define FAULT_ACTIVE(_m, _t) (drbd_insert_fault((_m), (_t)))
 
@@ -788,7 +788,13 @@ struct drbd_tl_epoch;
 struct drbd_request {
 	struct drbd_work w;
 	struct drbd_conf *mdev;
+
+	/* if local IO is not allowed, will be NULL.
+	 * if local IO _is_ allowed, holds the locally submitted bio clone,
+	 * or, after local IO completion, the ERR_PTR(error).
+	 * see drbd_endio_pri(). */
 	struct bio *private_bio;
+
 	struct hlist_node colision;
 	sector_t sector;
 	unsigned int size;
@@ -872,6 +878,11 @@ struct drbd_epoch_entry {
 	u64    block_id;
 };
 
+struct drbd_wq_barrier {
+	struct drbd_work w;
+	struct completion done;
+};
+
 struct digest_info {
 	int digest_size;
 	void *digest;
@@ -894,7 +905,7 @@ enum {
 	CREATE_BARRIER,		/* next P_DATA is preceeded by a P_BARRIER */
 	SIGNAL_ASENDER,		/* whether asender wants to be interrupted */
 	SEND_PING,		/* whether asender should send a ping asap */
-	WORK_PENDING,		/* completion flag for drbd_disconnect */
+
 	STOP_SYNC_TIMER,	/* tell timer to cancel itself */
 	UNPLUG_QUEUED,		/* only relevant with kernel 2.4 */
 	UNPLUG_REMOTE,		/* sending a "UnplugRemote" could help */
@@ -1158,7 +1169,11 @@ struct drbd_conf {
 	spinlock_t peer_seq_lock;
 	unsigned int minor;
 	unsigned long comm_bm_set; /* communicated number of set bits. */
-	cpumask_t cpu_mask;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,30) && !defined(cpumask_bits)
+	cpumask_t cpu_mask[1];
+#else
+	cpumask_var_t cpu_mask;
+#endif
 	struct bm_io_work bm_io_work;
 	u64 ed_uuid; /* UUID of the exposed data */
 	struct mutex state_mutex;
@@ -1233,10 +1248,10 @@ extern int  drbd_thread_start(struct drbd_thread *thi);
 extern void _drbd_thread_stop(struct drbd_thread *thi, int restart, int wait);
 #ifdef CONFIG_SMP
 extern void drbd_thread_current_set_cpu(struct drbd_conf *mdev);
-extern cpumask_t drbd_calc_cpu_mask(struct drbd_conf *mdev);
+extern void drbd_calc_cpu_mask(struct drbd_conf *mdev);
 #else
 #define drbd_thread_current_set_cpu(A) ({})
-#define drbd_calc_cpu_mask(A) CPU_MASK_ALL
+#define drbd_calc_cpu_mask(A) ({})
 #endif
 extern void drbd_free_resources(struct drbd_conf *mdev);
 extern void tl_release(struct drbd_conf *mdev, unsigned int barrier_nr,
@@ -1291,7 +1306,6 @@ extern int drbd_send_bitmap(struct drbd_conf *mdev);
 extern int _drbd_send_bitmap(struct drbd_conf *mdev);
 extern int drbd_send_sr_reply(struct drbd_conf *mdev, int retcode);
 extern void drbd_free_bc(struct drbd_backing_dev *ldev);
-extern int drbd_io_error(struct drbd_conf *mdev, int forcedetach);
 extern void drbd_mdev_cleanup(struct drbd_conf *mdev);
 
 /* drbd_meta-data.c (still in drbd_main.c) */
@@ -1458,7 +1472,7 @@ extern int  drbd_bm_set_bits(
 extern int  drbd_bm_clear_bits(
 		struct drbd_conf *mdev, unsigned long s, unsigned long e);
 /* bm_set_bits variant for use while holding drbd_bm_lock */
-extern int _drbd_bm_set_bits(struct drbd_conf *mdev,
+extern void _drbd_bm_set_bits(struct drbd_conf *mdev,
 		const unsigned long s, const unsigned long e);
 extern int  drbd_bm_test_bit(struct drbd_conf *mdev, unsigned long bitnr);
 extern int  drbd_bm_e_weight(struct drbd_conf *mdev, unsigned long enr);
@@ -1486,9 +1500,6 @@ extern void drbd_bm_get_lel(struct drbd_conf *mdev, size_t offset,
 extern void drbd_bm_lock(struct drbd_conf *mdev, char *why);
 extern void drbd_bm_unlock(struct drbd_conf *mdev);
 
-extern void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line);
-#define drbd_bm_recount_bits(mdev) \
-	_drbd_bm_recount_bits(mdev, __FILE__, __LINE__)
 extern int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, const unsigned long e);
 /* drbd_main.c */
 
@@ -1602,7 +1613,8 @@ extern void drbd_wait_ee_list_empty(struct drbd_conf *mdev,
 extern void _drbd_wait_ee_list_empty(struct drbd_conf *mdev,
 		struct list_head *head);
 extern void drbd_set_recv_tcq(struct drbd_conf *mdev, int tcq_enabled);
-extern void _drbd_clear_done_ee(struct drbd_conf *mdev);
+extern void _drbd_clear_done_ee(struct drbd_conf *mdev, struct list_head *to_be_freed);
+extern void drbd_flush_workqueue(struct drbd_conf *mdev);
 
 /* yes, there is kernel_setsockopt, but only since 2.6.18. we don't need to
  * mess with get_fs/set_fs, we know we are KERNEL_DS always. */
@@ -1651,8 +1663,8 @@ void drbd_bump_write_ordering(struct drbd_conf *mdev, enum write_ordering_e wo);
 /* drbd_proc.c */
 extern struct proc_dir_entry *drbd_proc;
 extern struct file_operations drbd_proc_fops;
-extern const char *conns_to_name(enum drbd_conns s);
-extern const char *roles_to_name(enum drbd_role s);
+extern const char *drbd_conn_str(enum drbd_conns s);
+extern const char *drbd_role_str(enum drbd_role s);
 
 /* drbd_actlog.c */
 extern void drbd_al_begin_io(struct drbd_conf *mdev, sector_t sector);
@@ -1793,13 +1805,15 @@ static inline int drbd_request_state(struct drbd_conf *mdev,
 	return _drbd_request_state(mdev, mask, val, CS_VERBOSE + CS_ORDERED);
 }
 
-static inline void __drbd_chk_io_error(struct drbd_conf *mdev, int forcedetach)
+#define __drbd_chk_io_error(m,f) __drbd_chk_io_error_(m,f, __func__)
+static inline void __drbd_chk_io_error_(struct drbd_conf *mdev, int forcedetach, const char *where)
 {
 	switch (mdev->ldev->dc.on_io_error) {
 	case EP_PASS_ON: /* FIXME would this be better named "Ignore"? */
 		if (!forcedetach) {
 			if (printk_ratelimit())
-				dev_err(DEV, "Local IO failed. Passing error on...\n");
+				dev_err(DEV, "Local IO failed in %s."
+					     "Passing error on...\n", where);
 			break;
 		}
 		/* NOTE fall through to detach case if forcedetach set */
@@ -1807,7 +1821,8 @@ static inline void __drbd_chk_io_error(struct drbd_conf *mdev, int forcedetach)
 	case EP_CALL_HELPER:
 		if (mdev->state.disk > D_FAILED) {
 			_drbd_set_state(_NS(mdev, disk, D_FAILED), CS_HARD, NULL);
-			dev_err(DEV, "Local IO failed. Detaching...\n");
+			dev_err(DEV, "Local IO failed in %s."
+				     "Detaching...\n", where);
 		}
 		break;
 	}
@@ -1819,15 +1834,16 @@ static inline void __drbd_chk_io_error(struct drbd_conf *mdev, int forcedetach)
  * @error:	 Error code passed to the IO completion callback
  * @forcedetach: Force detach. I.e. the error happened while accessing the meta data
  *
- * See also drbd_io_error().
+ * See also drbd_main.c:after_state_ch() if (os.disk > D_FAILED && ns.disk == D_FAILED)
  */
-static inline void drbd_chk_io_error(struct drbd_conf *mdev,
-	int error, int forcedetach)
+#define drbd_chk_io_error(m,e,f) drbd_chk_io_error_(m,e,f, __func__)
+static inline void drbd_chk_io_error_(struct drbd_conf *mdev,
+	int error, int forcedetach, const char *where)
 {
 	if (error) {
 		unsigned long flags;
 		spin_lock_irqsave(&mdev->req_lock, flags);
-		__drbd_chk_io_error(mdev, forcedetach);
+		__drbd_chk_io_error_(mdev, forcedetach, where);
 		spin_unlock_irqrestore(&mdev->req_lock, flags);
 	}
 }
@@ -2168,7 +2184,7 @@ static inline void drbd_get_syncer_progress(struct drbd_conf *mdev,
 		 */
 		smp_rmb();
 		dev_warn(DEV, "cs:%s rs_left=%lu > rs_total=%lu (rs_failed %lu)\n",
-				conns_to_name(mdev->state.conn),
+				drbd_conn_str(mdev->state.conn),
 				*bits_left, mdev->rs_total, mdev->rs_failed);
 		*per_mil_done = 0;
 	} else {
