@@ -217,24 +217,31 @@ enum drbd_disk_state drbd_try_outdate_peer(struct drbd_conf *mdev)
 		ex_to_string = "peer is inconsistent or worse";
 		nps = D_INCONSISTENT;
 		break;
-	case 4:
-		ex_to_string = "peer is outdated";
+	case 4: /* peer got outdated, or was already outdated */
+		ex_to_string = "peer was fenced";
 		nps = D_OUTDATED;
 		break;
-	case 5: /* peer was down, we will(have) create(d) a new UUID anyways... */
-		/* If we would be more strict, we would return D_UNKNOWN here. */
-		ex_to_string = "peer is unreachable, assumed to be dead";
-		nps = D_OUTDATED;
+	case 5: /* peer was down */
+		if (mdev->state.disk == D_UP_TO_DATE) {
+			/* we will(have) create(d) a new UUID anyways... */
+			ex_to_string = "peer is unreachable, assumed to be dead";
+			nps = D_OUTDATED;
+		} else {
+			ex_to_string = "peer unreachable, doing nothing since disk != UpToDate";
+			nps = mdev->state.pdsk;
+		}
 		break;
 	case 6: /* Peer is primary, voluntarily outdate myself.
 		 * This is useful when an unconnected R_SECONDARY is asked to
-		 * become R_PRIMARY, but findes the other peer being active. */
+		 * become R_PRIMARY, but finds the other peer being active. */
 		ex_to_string = "peer is active";
 		dev_warn(DEV, "Peer is primary, outdating myself.\n");
 		nps = D_UNKNOWN;
 		_drbd_request_state(mdev, NS(disk, D_OUTDATED), CS_WAIT_COMPLETE);
 		break;
 	case 7:
+		/* THINK: do we need to handle this
+		 * like case 4, or more like case 5? */
 		if (fp != FP_STONITH)
 			dev_err(DEV, "fence-peer() = 7 && fencing != Stonith !!!\n");
 		ex_to_string = "peer was stonithed";
@@ -292,11 +299,11 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 		}
 
 		if (r == SS_NO_UP_TO_DATE_DISK &&
-		    mdev->state.disk == D_CONSISTENT) {
+		    mdev->state.disk == D_CONSISTENT && mask.pdsk == 0) {
 			D_ASSERT(mdev->state.pdsk == D_UNKNOWN);
 			nps = drbd_try_outdate_peer(mdev);
 
-			if (nps == D_OUTDATED) {
+			if (nps == D_OUTDATED || nps == D_INCONSISTENT) {
 				val.disk = D_UP_TO_DATE;
 				mask.disk = D_MASK;
 			}
@@ -309,7 +316,7 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 
 		if (r == SS_NOTHING_TO_DO)
 			goto fail;
-		if (r == SS_PRIMARY_NOP) {
+		if (r == SS_PRIMARY_NOP && mask.pdsk == 0) {
 			nps = drbd_try_outdate_peer(mdev);
 
 			if (force && nps > D_OUTDATED) {
@@ -341,6 +348,9 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 		break;
 	}
 
+	if (r < SS_SUCCESS)
+		goto fail;
+
 	if (forced)
 		dev_warn(DEV, "Forced to consider local data as UpToDate!\n");
 
@@ -500,7 +510,7 @@ void drbd_resume_io(struct drbd_conf *mdev)
 }
 
 /**
- * drbd_determin_dev_size() -  Sets the right device size obeying all constraints
+ * drbd_determine_dev_size() -  Sets the right device size obeying all constraints
  * @mdev:	DRBD device.
  *
  * Returns 0 on success, negative return values indicate errors.
@@ -713,7 +723,7 @@ void drbd_setup_queue_param(struct drbd_conf *mdev, unsigned int max_seg_s) __mu
 	 * should be
 	 * t->max_segment_size = min_not_zero(...,...)
 	 * workaround here: */
-	if (queue_max_sectors(q) == 0)
+	if (queue_max_segment_size(q) == 0)
 		blk_queue_max_segment_size(q, max_seg_s);
 
 	if (b->merge_bvec_fn)
@@ -924,6 +934,8 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	drbd_suspend_io(mdev);
 	/* also wait for the last barrier ack. */
 	wait_event(mdev->misc_wait, !atomic_read(&mdev->ap_pending_cnt));
+	/* and for any other previously queued work */
+	drbd_flush_workqueue(mdev);
 
 	retcode = _drbd_request_state(mdev, NS(disk, D_ATTACHING), CS_VERBOSE);
 	drbd_resume_io(mdev);
@@ -987,7 +999,7 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 
 			dev_warn(DEV, "Meta data's bdev logical_block_size = %d != %d\n",
 			     logical_block_size, MD_SECTOR_SIZE);
-			dev_warn(DEV, "Workaround engaged (has performace impact).\n");
+			dev_warn(DEV, "Workaround engaged (has performance impact).\n");
 
 			mdev->md_io_tmpp = page;
 		}
@@ -1080,7 +1092,7 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	os = mdev->state;
 	ns.i = os.i;
 	/* If MDF_CONSISTENT is not set go into inconsistent state,
-	   otherwise investige MDF_WasUpToDate...
+	   otherwise investigate MDF_WasUpToDate...
 	   If MDF_WAS_UP_TO_DATE is not set go into D_OUTDATED disk state,
 	   otherwise into D_CONSISTENT state.
 	*/
@@ -1105,7 +1117,7 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	   this point, because drbd_request_state() modifies these
 	   flags. */
 
-	/* In case we are C_CONNECTED postpone any desicion on the new disk
+	/* In case we are C_CONNECTED postpone any decision on the new disk
 	   state after the negotiation phase. */
 	if (mdev->state.conn == C_CONNECTED) {
 		mdev->new_state_tmp.i = ns.i;
@@ -1448,7 +1460,7 @@ STATIC int drbd_nl_disconnect(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nl
 		retcode = drbd_request_state(mdev, NS2(conn, C_DISCONNECTING,
 						      pdsk, D_OUTDATED));
 	} else if (retcode == SS_CW_FAILED_BY_PEER) {
-		/* The peer probabely wants to see us outdated. */
+		/* The peer probably wants to see us outdated. */
 		retcode = _drbd_request_state(mdev, NS2(conn, C_DISCONNECTING,
 							disk, D_OUTDATED),
 					      CS_ORDERED);
@@ -1560,16 +1572,20 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 	struct crypto_hash *verify_tfm = NULL;
 	struct crypto_hash *csums_tfm = NULL;
 	struct syncer_conf sc;
-	cpumask_t n_cpu_mask = CPU_MASK_NONE;
+	cpumask_var_t new_cpu_mask;
 
-	memcpy(&sc, &mdev->sync_conf, sizeof(struct syncer_conf));
+	if (!zalloc_cpumask_var(&new_cpu_mask, GFP_KERNEL)) {
+		retcode = ERR_NOMEM;
+		goto fail;
+	}
 
 	if (nlp->flags & DRBD_NL_SET_DEFAULTS) {
 		memset(&sc, 0, sizeof(struct syncer_conf));
 		sc.rate       = DRBD_RATE_DEF;
 		sc.after      = DRBD_AFTER_DEF;
 		sc.al_extents = DRBD_AL_EXTENTS_DEF;
-	}
+	} else
+		memcpy(&sc, &mdev->sync_conf, sizeof(struct syncer_conf));
 
 	if (!syncer_conf_from_tags(mdev, nlp->tag_list, &sc)) {
 		retcode = ERR_MANDATORY_TAG;
@@ -1626,8 +1642,9 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 	}
 
 	/* silently ignore cpu mask on UP kernel */
-	if (NR_CPUS > 1 && sc.cpu_mask[0] != 0) {
-		err = __bitmap_parse(sc.cpu_mask, 32, 0, (unsigned long *)&n_cpu_mask, NR_CPUS);
+	if (nr_cpu_ids > 1 && sc.cpu_mask[0] != 0) {
+		err = __bitmap_parse(sc.cpu_mask, 32, 0,
+				cpumask_bits(new_cpu_mask), nr_cpu_ids);
 		if (err) {
 			dev_warn(DEV, "__bitmap_parse() failed with %d\n", err);
 			retcode = ERR_CPU_MASK_PARSE;
@@ -1688,9 +1705,9 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 	if (mdev->state.conn >= C_CONNECTED)
 		drbd_send_sync_param(mdev, &sc);
 
-	if (!cpus_equal(mdev->cpu_mask, n_cpu_mask)) {
-		mdev->cpu_mask = n_cpu_mask;
-		mdev->cpu_mask = drbd_calc_cpu_mask(mdev);
+	if (!cpumask_equal(mdev->cpu_mask, new_cpu_mask)) {
+		cpumask_copy(mdev->cpu_mask, new_cpu_mask);
+		drbd_calc_cpu_mask(mdev);
 		mdev->receiver.reset_cpu_mask = 1;
 		mdev->asender.reset_cpu_mask = 1;
 		mdev->worker.reset_cpu_mask = 1;
@@ -1698,6 +1715,7 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 
 	drbd_kobject_uevent(mdev);
 fail:
+	free_cpumask_var(new_cpu_mask);
 	crypto_free_hash(csums_tfm);
 	crypto_free_hash(verify_tfm);
 	reply->ret_code = retcode;
@@ -2022,9 +2040,14 @@ static struct cn_handler_struct cnd_table[] = {
 	[ P_new_c_uuid ]	= { &drbd_nl_new_c_uuid,	0 },
 };
 
+#ifdef KERNEL_HAS_CN_SKB_PARMS
+STATIC void drbd_connector_callback(struct cn_msg *req, struct netlink_skb_parms *nsp)
+{
+#else
 STATIC void drbd_connector_callback(void *data)
 {
 	struct cn_msg *req = data;
+#endif
 	struct drbd_nl_cfg_req *nlp = (struct drbd_nl_cfg_req *)req->data;
 	struct cn_handler_struct *cm;
 	struct cn_msg *cn_reply;
@@ -2040,13 +2063,20 @@ STATIC void drbd_connector_callback(void *data)
 		return;
 	}
 
+#ifdef KERNEL_HAS_CN_SKB_PARMS
+	if (!cap_raised(nsp->eff_cap, CAP_SYS_ADMIN)) {
+		retcode = ERR_PERM;
+		goto fail;
+	}
+#endif
+
 	mdev = ensure_mdev(nlp);
 	if (!mdev) {
 		retcode = ERR_MINOR_INVALID;
 		goto fail;
 	}
 
-	trace_drbd_netlink(data, 1);
+	trace_drbd_netlink(req, 1);
 
 	if (nlp->packet_type >= P_nl_after_last_packet) {
 		retcode = ERR_PACKET_NR;
@@ -2075,7 +2105,7 @@ STATIC void drbd_connector_callback(void *data)
 		cm->reply_body_size ? nlp->packet_type : P_nl_after_last_packet;
 	reply->minor = nlp->drbd_minor;
 	reply->ret_code = NO_ERROR; /* Might by modified by cm->function. */
-	/* reply->tag_list; might be modified by cm->fucntion. */
+	/* reply->tag_list; might be modified by cm->function. */
 
 	rr = cm->function(mdev, nlp, reply);
 
@@ -2134,12 +2164,12 @@ tl_add_int(unsigned short *tl, enum drbd_tags tag, const void *val)
 	switch(tag_type(tag)) {
 	case TT_INTEGER:
 		put_unaligned(sizeof(int), tl++);
-		put_unaligned(*(int *)val, (int *)tl++);
+		put_unaligned(*(int *)val, (int *)tl);
 		tl = (unsigned short*)((char*)tl+sizeof(int));
 		break;
 	case TT_INT64:
 		put_unaligned(sizeof(u64), tl++);
-		put_unaligned(*(u64 *)val, (u64 *)tl++);
+		put_unaligned(*(u64 *)val, (u64 *)tl);
 		tl = (unsigned short*)((char*)tl+sizeof(u64));
 		break;
 	default:
@@ -2180,7 +2210,7 @@ void drbd_bcast_state(struct drbd_conf *mdev, union drbd_state state)
 	reply->ret_code = NO_ERROR;
 
 	trace_drbd_netlink(cn_reply, 0);
-	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
+	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_NOIO);
 }
 
 void drbd_bcast_ev_helper(struct drbd_conf *mdev, char *helper_name)
@@ -2213,7 +2243,7 @@ void drbd_bcast_ev_helper(struct drbd_conf *mdev, char *helper_name)
 	reply->ret_code = NO_ERROR;
 
 	trace_drbd_netlink(cn_reply, 0);
-	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
+	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_NOIO);
 }
 
 void drbd_bcast_ee(struct drbd_conf *mdev,
@@ -2232,7 +2262,7 @@ void drbd_bcast_ee(struct drbd_conf *mdev,
 	if (!reason || !reason[0])
 		return;
 
-	/* aparently we have to memcpy twice, first to prepare the data for the
+	/* apparently we have to memcpy twice, first to prepare the data for the
 	 * struct cn_msg, then within cn_netlink_send from the cn_msg to the
 	 * netlink skb. */
 	/* receiver thread context, which is not in the writeout path (of this node),
@@ -2285,7 +2315,7 @@ void drbd_bcast_ee(struct drbd_conf *mdev,
 	reply->ret_code = NO_ERROR;
 
 	trace_drbd_netlink(cn_reply, 0);
-	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
+	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_NOIO);
 	kfree(cn_reply);
 }
 
@@ -2325,7 +2355,7 @@ void drbd_bcast_sync_progress(struct drbd_conf *mdev)
 	reply->ret_code = NO_ERROR;
 
 	trace_drbd_netlink(cn_reply, 0);
-	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
+	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_NOIO);
 }
 
 #ifdef NETLINK_ROUTE6
@@ -2395,7 +2425,7 @@ void drbd_nl_send_reply(struct cn_msg *req, int ret_code)
 	reply->ret_code = ret_code;
 
 	trace_drbd_netlink(cn_reply, 0);
-	rr = cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
+	rr = cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_NOIO);
 	if (rr && rr != -ESRCH)
 		printk(KERN_INFO "drbd: cn_netlink_send()=%d\n", rr);
 }
