@@ -64,7 +64,7 @@ STATIC int w_make_ov_request(struct drbd_conf *mdev, struct drbd_work *w, int ca
    atodb_endio in drbd_actlog.c
    drbd_bm_async_io_complete in drbd_bitmap.c
 
- * For all these callbacks, note the follwing:
+ * For all these callbacks, note the following:
  * The callbacks will be called in irq context by the IDE drivers,
  * and in Softirqs/Tasklets/BH context by the SCSI drivers.
  * Try to get the locking right :)
@@ -110,11 +110,15 @@ BIO_ENDIO_TYPE drbd_endio_read_sec BIO_ENDIO_ARGS(struct bio *bio, int error) __
 	mdev = e->mdev;
 
 	BIO_ENDIO_FN_START;
+	if (error)
+		dev_warn(DEV, "read: error=%d s=%llus\n", error,
+				(unsigned long long)e->sector);
 	if (!error && !uptodate) {
-		/* strange behaviour of some lower level drivers...
+		dev_warn(DEV, "read: setting error to -EIO s=%llus\n",
+				(unsigned long long)e->sector);
+		/* strange behavior of some lower level drivers...
 		 * fail the request by clearing the uptodate flag,
-		 * but do not return any error?!
-		 * do we want to dev_warn(DEV, ) on this? */
+		 * but do not return any error?! */
 		error = -EIO;
 	}
 
@@ -150,27 +154,33 @@ BIO_ENDIO_TYPE drbd_endio_write_sec BIO_ENDIO_ARGS(struct bio *bio, int error) _
 	int is_syncer_req;
 	int do_al_complete_io;
 	int uptodate = bio_flagged(bio, BIO_UPTODATE);
+	int is_barrier = bio_rw_flagged(bio, BIO_RW_BARRIER);
 
 	e = bio->bi_private;
 	mdev = e->mdev;
 
 	BIO_ENDIO_FN_START;
+	if (error)
+		dev_warn(DEV, "write: error=%d s=%llus\n", error,
+				(unsigned long long)e->sector);
 	if (!error && !uptodate) {
-		/* strange behaviour of some lower level drivers...
+		dev_warn(DEV, "write: setting error to -EIO s=%llus\n",
+				(unsigned long long)e->sector);
+		/* strange behavior of some lower level drivers...
 		 * fail the request by clearing the uptodate flag,
-		 * but do not return any error?!
-		 * do we want to dev_warn(DEV, ) on this? */
+		 * but do not return any error?! */
 		error = -EIO;
 	}
 
 	/* error == -ENOTSUPP would be a better test,
 	 * alas it is not reliable */
-	if (error && e->flags & EE_IS_BARRIER) {
+	if (error && is_barrier && e->flags & EE_IS_BARRIER) {
 		drbd_bump_write_ordering(mdev, WO_bdev_flush);
 		spin_lock_irqsave(&mdev->req_lock, flags);
 		list_del(&e->w.list);
 		e->w.cb = w_e_reissue;
-		__release(local); /* Actually happens in w_e_reissue. */
+		/* put_ldev actually happens below, once we come here again. */
+		__release(local);
 		spin_unlock_irqrestore(&mdev->req_lock, flags);
 		drbd_queue_work(&mdev->data.work, &e->w);
 		BIO_ENDIO_FN_RETURN;
@@ -224,43 +234,57 @@ BIO_ENDIO_TYPE drbd_endio_write_sec BIO_ENDIO_ARGS(struct bio *bio, int error) _
 	BIO_ENDIO_FN_RETURN;
 }
 
-/* read, readA or write requests on R_PRIMARY comming from drbd_make_request
+/* read, readA or write requests on R_PRIMARY coming from drbd_make_request
  */
 BIO_ENDIO_TYPE drbd_endio_pri BIO_ENDIO_ARGS(struct bio *bio, int error)
 {
 	unsigned long flags;
 	struct drbd_request *req = bio->bi_private;
 	struct drbd_conf *mdev = req->mdev;
+	struct bio_and_error m;
 	enum drbd_req_event what;
 	int uptodate = bio_flagged(bio, BIO_UPTODATE);
 
 	BIO_ENDIO_FN_START;
+	if (error)
+		dev_warn(DEV, "p %s: error=%d\n",
+			 bio_data_dir(bio) == WRITE ? "write" : "read", error);
 	if (!error && !uptodate) {
-		/* strange behaviour of some lower level drivers...
+		dev_warn(DEV, "p %s: setting error to -EIO\n",
+			 bio_data_dir(bio) == WRITE ? "write" : "read");
+		/* strange behavior of some lower level drivers...
 		 * fail the request by clearing the uptodate flag,
-		 * but do not return any error?!
-		 * do we want to dev_warn(DEV, ) on this? */
+		 * but do not return any error?! */
 		error = -EIO;
 	}
 
 	trace_drbd_bio(mdev, "Pri", bio, 1, NULL);
 
-	/* to avoid recursion in _req_mod */
-	what = error
-	       ? (bio_data_dir(bio) == WRITE)
-		 ? write_completed_with_error
-		 : read_completed_with_error
-	       : completed_ok;
+	/* to avoid recursion in __req_mod */
+	if (unlikely(error)) {
+		what = (bio_data_dir(bio) == WRITE)
+			? write_completed_with_error
+			: (bio_rw(bio) == READA)
+			  ? read_completed_with_error
+			  : read_ahead_completed_with_error;
+	} else
+		what = completed_ok;
+
+	bio_put(req->private_bio);
+	req->private_bio = ERR_PTR(error);
+
 	spin_lock_irqsave(&mdev->req_lock, flags);
-	_req_mod(req, what, error);
+	__req_mod(req, what, &m);
 	spin_unlock_irqrestore(&mdev->req_lock, flags);
+
+	if (m.bio)
+		complete_master_bio(mdev, &m);
 	BIO_ENDIO_FN_RETURN;
 }
 
 int w_io_error(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_request *req = (struct drbd_request *)w;
-	int ok;
+	struct drbd_request *req = container_of(w, struct drbd_request, w);
 
 	/* NOTE: mdev->ldev can be NULL by the time we get here! */
 	/* D_ASSERT(mdev->ldev->dc.on_io_error != EP_PASS_ON); */
@@ -269,26 +293,22 @@ int w_io_error(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	 * when it is done and had a local write error, see comments there */
 	drbd_req_free(req);
 
-	ok = drbd_io_error(mdev, FALSE);
-	if (unlikely(!ok))
-		dev_err(DEV, "Sending in w_io_error() failed\n");
-	return ok;
+	return TRUE;
 }
 
 int w_read_retry_remote(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_request *req = (struct drbd_request *)w;
+	struct drbd_request *req = container_of(w, struct drbd_request, w);
 
 	/* We should not detach for read io-error,
 	 * but try to WRITE the P_DATA_REPLY to the failed location,
 	 * to give the disk the chance to relocate that block */
-	drbd_io_error(mdev, FALSE); /* tries to schedule a detach and notifies peer */
 
 	spin_lock_irq(&mdev->req_lock);
 	if (cancel ||
 	    mdev->state.conn < C_CONNECTED ||
 	    mdev->state.pdsk <= D_INCONSISTENT) {
-		_req_mod(req, send_canceled, 0);
+		_req_mod(req, send_canceled);
 		spin_unlock_irq(&mdev->req_lock);
 		dev_alert(DEV, "WE ARE LOST. Local IO failure, no peer.\n");
 		return 1;
@@ -327,7 +347,7 @@ void drbd_csum(struct drbd_conf *mdev, struct crypto_hash *tfm, struct bio *bio,
 
 STATIC int w_e_send_csum(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_epoch_entry *e = (struct drbd_epoch_entry *)w;
+	struct drbd_epoch_entry *e = container_of(w, struct drbd_epoch_entry, w);
 	int digest_size;
 	void *digest;
 	int ok;
@@ -357,10 +377,8 @@ STATIC int w_e_send_csum(struct drbd_conf *mdev, struct drbd_work *w, int cancel
 			dev_err(DEV, "kmalloc() of digest failed.\n");
 			ok = 0;
 		}
-	} else {
-		drbd_io_error(mdev, FALSE);
+	} else
 		ok = 1;
-	}
 
 	drbd_free_ee(mdev, e);
 
@@ -378,9 +396,8 @@ STATIC int read_for_csum(struct drbd_conf *mdev, sector_t sector, int size)
 	if (!get_ldev(mdev))
 		return 0;
 
-	if (FAULT_ACTIVE(mdev, DRBD_FAULT_AL_EE))
-		return 2;
-
+	/* GFP_TRY, because if there is no memory available right now, this may
+	 * be rescheduled for later. It is "only" background resync, after all. */
 	e = drbd_alloc_ee(mdev, DRBD_MAGIC+0xbeef, sector, size, GFP_TRY);
 	if (!e) {
 		put_ldev(mdev);
@@ -449,7 +466,7 @@ int w_make_resync_request(struct drbd_conf *mdev,
 
 	if (mdev->state.conn != C_SYNC_TARGET)
 		dev_err(DEV, "%s in w_make_resync_request\n",
-			conns_to_name(mdev->state.conn));
+			drbd_conn_str(mdev->state.conn));
 
 	if (!get_ldev(mdev)) {
 		/* Since we only need to access mdev->rsync a
@@ -475,13 +492,13 @@ int w_make_resync_request(struct drbd_conf *mdev,
 	if (number > mx)
 		mx = number;
 
-	/* Limit the nunber of pending RS requests to no more than the peer's receive buffer */
+	/* Limit the number of pending RS requests to no more than the peer's receive buffer */
 	if ((pe + number) > mx) {
 		number = mx - pe;
 	}
 
 	for (i = 0; i < number; i++) {
-		/* Stop generating RS requests, when half of the sendbuffer is filled */
+		/* Stop generating RS requests, when half of the send buffer is filled */
 		mutex_lock(&mdev->data.mutex);
 		if (mdev->data.socket) {
 			queued = mdev->data.socket->sk->sk_wmem_queued;
@@ -522,10 +539,10 @@ next_sector:
 		/* try to find some adjacent bits.
 		 * we stop if we have already the maximum req size.
 		 *
-		 * Aditionally always align bigger requests, in order to
+		 * Additionally always align bigger requests, in order to
 		 * be prepared for all stripe sizes of software RAIDs.
 		 *
-		 * we _do_ care about the agreed-uppon q->max_segment_size
+		 * we _do_ care about the agreed-upon q->max_segment_size
 		 * here, as splitting up the requests on the other side is more
 		 * difficult.  the consequence is, that on lvm and md and other
 		 * "indirect" devices, this is dead code, since
@@ -606,7 +623,7 @@ next_sector:
 	return 1;
 }
 
-int w_make_ov_request(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
+STATIC int w_make_ov_request(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
 	int number, i, size;
 	sector_t sector;
@@ -688,7 +705,7 @@ int drbd_resync_finished(struct drbd_conf *mdev)
 	 * might set bits in the (main) bitmap, then the entries in the
 	 * resync LRU would be wrong. */
 	if (drbd_rs_del_all(mdev)) {
-		/* In case this is not possible now, most probabely because
+		/* In case this is not possible now, most probably because
 		 * there are P_RS_DATA_REPLY Packets lingering on the worker's
 		 * queue (or even the read operations for those packets
 		 * is not finished by now).   Retry in 100ms. */
@@ -813,14 +830,24 @@ out:
 		drbd_queue_bitmap_io(mdev, &drbd_bm_write, NULL, "write from resync_finished");
 	}
 
-	drbd_bm_recount_bits(mdev);
-
 	if (khelper_cmd)
 		drbd_khelper(mdev, khelper_cmd);
 
 	return 1;
 }
 
+/* helper */
+static void move_to_net_ee_or_free(struct drbd_conf *mdev, struct drbd_epoch_entry *e)
+{
+	if (drbd_bio_has_active_page(e->private_bio)) {
+		/* This might happen if sendpage() has not finished */
+		spin_lock_irq(&mdev->req_lock);
+		list_add_tail(&e->w.list, &mdev->net_ee);
+		spin_unlock_irq(&mdev->req_lock);
+	} else
+		drbd_free_ee(mdev, e);
+}
+
 /**
  * w_e_end_data_req() - Worker callback, to send a P_DATA_REPLY packet in response to a P_DATA_REQUEST
  * @mdev:	DRBD device.
@@ -829,7 +856,7 @@ out:
  */
 int w_e_end_data_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_epoch_entry *e = (struct drbd_epoch_entry *)w;
+	struct drbd_epoch_entry *e = container_of(w, struct drbd_epoch_entry, w);
 	int ok;
 
 	if (unlikely(cancel)) {
@@ -846,20 +873,11 @@ int w_e_end_data_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 			    (unsigned long long)e->sector);
 
 		ok = drbd_send_ack(mdev, P_NEG_DREPLY, e);
-
-		drbd_io_error(mdev, FALSE);
 	}
 
 	dec_unacked(mdev);
 
-	spin_lock_irq(&mdev->req_lock);
-	if (drbd_bio_has_active_page(e->private_bio)) {
-		/* This might happen if sendpage() has not finished */
-		list_add_tail(&e->w.list, &mdev->net_ee);
-	} else {
-		drbd_free_ee(mdev, e);
-	}
-	spin_unlock_irq(&mdev->req_lock);
+	move_to_net_ee_or_free(mdev, e);
 
 	if (unlikely(!ok))
 		dev_err(DEV, "drbd_send_block() failed\n");
@@ -874,7 +892,7 @@ int w_e_end_data_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
  */
 int w_e_end_rsdata_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_epoch_entry *e = (struct drbd_epoch_entry *)w;
+	struct drbd_epoch_entry *e = container_of(w, struct drbd_epoch_entry, w);
 	int ok;
 
 	if (unlikely(cancel)) {
@@ -905,22 +923,13 @@ int w_e_end_rsdata_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 
 		ok = drbd_send_ack(mdev, P_NEG_RS_DREPLY, e);
 
-		drbd_io_error(mdev, FALSE);
-
 		/* update resync data with failure */
 		drbd_rs_failed_io(mdev, e->sector, e->size);
 	}
 
 	dec_unacked(mdev);
 
-	spin_lock_irq(&mdev->req_lock);
-	if (drbd_bio_has_active_page(e->private_bio)) {
-		/* This might happen if sendpage() has not finished */
-		list_add_tail(&e->w.list, &mdev->net_ee);
-	} else {
-		drbd_free_ee(mdev, e);
-	}
-	spin_unlock_irq(&mdev->req_lock);
+	move_to_net_ee_or_free(mdev, e);
 
 	if (unlikely(!ok))
 		dev_err(DEV, "drbd_send_block() failed\n");
@@ -929,7 +938,7 @@ int w_e_end_rsdata_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 
 int w_e_end_csum_rs_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_epoch_entry *e = (struct drbd_epoch_entry *)w;
+	struct drbd_epoch_entry *e = container_of(w, struct drbd_epoch_entry, w);
 	struct digest_info *di;
 	int digest_size;
 	void *digest = NULL;
@@ -973,21 +982,13 @@ int w_e_end_csum_rs_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 		ok = drbd_send_ack(mdev, P_NEG_RS_DREPLY, e);
 		if (DRBD_ratelimit(5*HZ, 5))
 			dev_err(DEV, "Sending NegDReply. I guess it gets messy.\n");
-		drbd_io_error(mdev, FALSE);
 	}
 
 	dec_unacked(mdev);
 
 	kfree(di);
 
-	spin_lock_irq(&mdev->req_lock);
-	if (drbd_bio_has_active_page(e->private_bio)) {
-		/* This might happen if sendpage() has not finished */
-		list_add_tail(&e->w.list, &mdev->net_ee);
-	} else {
-		drbd_free_ee(mdev, e);
-	}
-	spin_unlock_irq(&mdev->req_lock);
+	move_to_net_ee_or_free(mdev, e);
 
 	if (unlikely(!ok))
 		dev_err(DEV, "drbd_send_block/ack() failed\n");
@@ -996,7 +997,7 @@ int w_e_end_csum_rs_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 
 int w_e_end_ov_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_epoch_entry *e = (struct drbd_epoch_entry *)w;
+	struct drbd_epoch_entry *e = container_of(w, struct drbd_epoch_entry, w);
 	int digest_size;
 	void *digest;
 	int ok = 1;
@@ -1021,9 +1022,7 @@ int w_e_end_ov_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	}
 
 out:
-	spin_lock_irq(&mdev->req_lock);
 	drbd_free_ee(mdev, e);
-	spin_unlock_irq(&mdev->req_lock);
 
 	dec_unacked(mdev);
 
@@ -1044,7 +1043,7 @@ void drbd_ov_oos_found(struct drbd_conf *mdev, sector_t sector, int size)
 
 int w_e_end_ov_reply(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_epoch_entry *e = (struct drbd_epoch_entry *)w;
+	struct drbd_epoch_entry *e = container_of(w, struct drbd_epoch_entry, w);
 	struct digest_info *di;
 	int digest_size;
 	void *digest;
@@ -1076,7 +1075,6 @@ int w_e_end_ov_reply(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 		ok = drbd_send_ack(mdev, P_NEG_RS_DREPLY, e);
 		if (DRBD_ratelimit(5*HZ, 5))
 			dev_err(DEV, "Sending NegDReply. I guess it gets messy.\n");
-		drbd_io_error(mdev, FALSE);
 	}
 
 	dec_unacked(mdev);
@@ -1091,9 +1089,7 @@ int w_e_end_ov_reply(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	ok = drbd_send_ack_ex(mdev, P_OV_RESULT, e->sector, e->size,
 			      eq ? ID_IN_SYNC : ID_OUT_OF_SYNC);
 
-	spin_lock_irq(&mdev->req_lock);
 	drbd_free_ee(mdev, e);
-	spin_unlock_irq(&mdev->req_lock);
 
 	if (--mdev->ov_left == 0) {
 		ov_oos_print(mdev);
@@ -1105,19 +1101,19 @@ int w_e_end_ov_reply(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 
 int w_prev_work_done(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	clear_bit(WORK_PENDING, &mdev->flags);
-	wake_up(&mdev->misc_wait);
+	struct drbd_wq_barrier *b = container_of(w, struct drbd_wq_barrier, w);
+	complete(&b->done);
 	return 1;
 }
 
 int w_send_barrier(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_tl_epoch *b = (struct drbd_tl_epoch *)w;
+	struct drbd_tl_epoch *b = container_of(w, struct drbd_tl_epoch, w);
 	struct p_barrier *p = &mdev->data.sbuf.barrier;
 	int ok = 1;
 
 	/* really avoid racing with tl_clear.  w.cb may have been referenced
-	 * just before it was reassigned and requeued, so double check that.
+	 * just before it was reassigned and re-queued, so double check that.
 	 * actually, this race was harmless, since we only try to send the
 	 * barrier packet here, and otherwise do nothing with the object.
 	 * but compare with the head of w_clear_epoch */
@@ -1156,16 +1152,16 @@ int w_send_write_hint(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
  */
 int w_send_dblock(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_request *req = (struct drbd_request *)w;
+	struct drbd_request *req = container_of(w, struct drbd_request, w);
 	int ok;
 
 	if (unlikely(cancel)) {
-		req_mod(req, send_canceled, 0);
+		req_mod(req, send_canceled);
 		return 1;
 	}
 
 	ok = drbd_send_dblock(mdev, req);
-	req_mod(req, ok ? handed_over_to_network : send_failed, 0);
+	req_mod(req, ok ? handed_over_to_network : send_failed);
 
 	return ok;
 }
@@ -1178,11 +1174,11 @@ int w_send_dblock(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
  */
 int w_send_read_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
-	struct drbd_request *req = (struct drbd_request *)w;
+	struct drbd_request *req = container_of(w, struct drbd_request, w);
 	int ok;
 
 	if (unlikely(cancel)) {
-		req_mod(req, send_canceled, 0);
+		req_mod(req, send_canceled);
 		return 1;
 	}
 
@@ -1195,7 +1191,7 @@ int w_send_read_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 		if (mdev->state.conn >= C_CONNECTED)
 			drbd_force_state(mdev, NS(conn, C_NETWORK_FAILURE));
 	}
-	req_mod(req, ok ? handed_over_to_network : send_failed, 0);
+	req_mod(req, ok ? handed_over_to_network : send_failed);
 
 	return ok;
 }
@@ -1346,8 +1342,6 @@ void drbd_start_resync(struct drbd_conf *mdev, enum drbd_conns side)
 	trace_drbd_resync(mdev, TRACE_LVL_SUMMARY, "Resync starting: side=%s\n",
 			  side == C_SYNC_TARGET ? "SyncTarget" : "SyncSource");
 
-	drbd_bm_recount_bits(mdev);
-
 	/* In case a previous resync run was aborted by an IO error/detach on the peer. */
 	drbd_rs_cancel_all(mdev);
 
@@ -1419,11 +1413,15 @@ void drbd_start_resync(struct drbd_conf *mdev, enum drbd_conns side)
 
 	if (r == SS_SUCCESS) {
 		dev_info(DEV, "Began resync as %s (will sync %lu KB [%lu bits set]).\n",
-		     conns_to_name(ns.conn),
+		     drbd_conn_str(ns.conn),
 		     (unsigned long) mdev->rs_total << (BM_BLOCK_SHIFT-10),
 		     (unsigned long) mdev->rs_total);
 
 		if (mdev->rs_total == 0) {
+			/* Peer still reachable? Beware of failing before-resync-target handlers! */
+			request_ping(mdev);
+			__set_current_state(TASK_INTERRUPTIBLE);
+			schedule_timeout(mdev->net_conf->ping_timeo*HZ/9); /* 9 instead 10 */
 			drbd_resync_finished(mdev);
 			return;
 		}
