@@ -73,7 +73,7 @@ struct adm_cmd {
 	 * 0: don't show this command, ever
 	 * 1: normal administrative commands, shown in normal help
 	 * 2-4: shown on "drbdadm hidden-commands"
-	 * 2: usefull for shell scripts
+	 * 2: useful for shell scripts
 	 * 3: callbacks potentially called from kernel module on certain events
 	 * 4: advanced, experts and developers only */
 	unsigned int show_in_usage:3;
@@ -165,7 +165,7 @@ int dry_run = 0;
 int verbose = 0;
 int do_verify_ips = 0;
 int do_register_minor = 1;
-/* wether drbdadm was called with "all" instead of resource name(s) */
+/* whether drbdadm was called with "all" instead of resource name(s) */
 int all_resources = 0;
 char *drbdsetup = NULL;
 char *drbdmeta = NULL;
@@ -290,7 +290,7 @@ enum do_register if_conf_differs_confirm_or_abort(struct d_resource *res)
 
 	if (!confirmed("Do you want to proceed "
 		       "and register the current config file?")) {
-		printf("Operation cancelled.\n");
+		printf("Operation canceled.\n");
 		exit(E_usage);
 	}
 	return DO_REGISTER;
@@ -333,8 +333,8 @@ int call_cmd_fn(int (*function) (struct d_resource *, const char *),
 int call_cmd(struct adm_cmd *cmd, struct d_resource *res,
 	     enum on_error on_error)
 {
-	if (!res->peer && cmd->need_peer)
-		set_peer_in_resource(res, 1);
+	if (!res->peer)
+		set_peer_in_resource(res, cmd->need_peer);
 
 	return call_cmd_fn(cmd->function, cmd->name, res, on_error);
 }
@@ -1610,24 +1610,38 @@ static int adm_khelper(struct d_resource *res, const char *cmd)
 {
 	int rv = 0;
 	char *sh_cmd;
+	char minor_string[8];
 	char *argv[] = { "/bin/sh", "-c", NULL, NULL };
 
-	/* res->peer is not available here if one has more than two host sections in his
-	   resoruce definition.
-	   Therefore DRBD_PEERS will not be available. */
+	if (!res->peer) {
+		/* Since 8.3.2 we get DRBD_PEER_AF and DRBD_PEER_ADDRESS from the kernel.
+		   If we do not know the peer by now, use these to find the peer. */
+		struct d_host_info *host;
+		char *peer_address = getenv("DRBD_PEER_ADDRESS");
+		char *peer_af = getenv("DRBD_PEER_AF");
+
+		if (peer_address && peer_af) {
+			for (host = res->all_hosts; host; host = host->next) {
+				if (!strcmp(host->address_family, peer_af) &&
+				    !strcmp(host->address, peer_address)) {
+					res->peer = host;
+					break;
+				}
+			}
+		}
+	}
 
-	/* Since 8.3.2 we get DRBD_PEER_AF and DRBD_PEER_ADDRESS from the kernel.
-	   If we know the peer anyways (i.e. not more than two host sections) we
-	   set it (overwrite) it here anyways. */
 	if (res->peer) {
 		setenv("DRBD_PEER_AF", res->peer->address_family, 1);	/* since 8.3.0 */
 		setenv("DRBD_PEER_ADDRESS", res->peer->address, 1);	/* since 8.3.0 */
 		setenv("DRBD_PEER", res->peer->on_hosts->name, 1);	/* deprecated */
 		setenv("DRBD_PEERS", names_to_str(res->peer->on_hosts), 1);
-			/* since 8.3.0, but not useable when using a config with "floating" statements. */
+			/* since 8.3.0, but not usable when using a config with "floating" statements. */
 	}
 
+	snprintf(minor_string, sizeof(minor_string), "%u", res->me->device_minor);
 	setenv("DRBD_RESOURCE", res->name, 1);
+	setenv("DRBD_MINOR", minor_string, 1);
 	setenv("DRBD_CONF", config_save, 1);
 
 	if ((sh_cmd = get_opt_val(res->handlers, cmd, NULL))) {
@@ -1871,8 +1885,8 @@ static int adm_up(struct d_resource *res,
 	return 0;
 }
 
-/* The stacked-timeouts switch in the startup sections allowes us
-   to enforce the use of the specified timeouts insetead the use
+/* The stacked-timeouts switch in the startup sections allows us
+   to enforce the use of the specified timeouts instead the use
    of a sane value. Should only be used if the third node should
    never become primary. */
 static int adm_wait_c(struct d_resource *res,
@@ -1889,7 +1903,7 @@ static int adm_wait_c(struct d_resource *res,
 		unsigned long timeout = 20;
 		if ((opt = find_opt(res->net_options, "connect-int"))) {
 			timeout = strtoul(opt->value, NULL, 10);
-			// one connect-intervall? two?
+			// one connect-interval? two?
 			timeout *= 2;
 		}
 		argv[argc++] = "-t";
@@ -1906,20 +1920,20 @@ static int adm_wait_c(struct d_resource *res,
 	return rv;
 }
 
-static int minor_by_id(const char *id)
+static unsigned minor_by_id(const char *id)
 {
 	if (strncmp(id, "minor-", 6))
-		return -1;
+		return -1U;
 	return m_strtoll(id + 6, 1);
 }
 
 struct d_resource *res_by_minor(const char *id)
 {
 	struct d_resource *res, *t;
-	int mm;
+	unsigned int mm;
 
 	mm = minor_by_id(id);
-	if (mm < 0)
+	if (mm == -1U)
 		return NULL;
 
 	for_each_resource(res, t, config) {
@@ -2301,7 +2315,7 @@ static struct ifreq *get_ifreq(void)
 	} while (buf_size <= (size_t) ifc.ifc_len);
 
 	num_ifaces = ifc.ifc_len / sizeof(struct ifreq);
-	/* Since we allocated at least one more than neccessary,
+	/* Since we allocated at least one more than necessary,
 	 * this serves as a stop marker for the iteration in
 	 * have_ip() */
 	ifc.ifc_req[num_ifaces].ifr_name[0] = 0;
@@ -2547,15 +2561,6 @@ void validate_resource(struct d_resource *res)
 	} else {
 		res->protocol[0] = toupper(res->protocol[0]);
 	}
-	if (res->ignore)
-		return;
-	if (!res->me) {
-		fprintf(stderr,
-			"%s:%d: in resource %s:\n\tmissing section 'on %s { ... }'.\n",
-			res->config_file, res->start_line, res->name,
-			nodeinfo.nodename);
-		config_valid = 0;
-	}
 	if ((opt = find_opt(res->sync_options, "after"))) {
 		if (res_by_name(opt->value) == NULL) {
 			fprintf(stderr,
@@ -2566,6 +2571,15 @@ void validate_resource(struct d_resource *res)
 			config_valid = 0;
 		}
 	}
+	if (res->ignore)
+		return;
+	if (!res->me) {
+		fprintf(stderr,
+			"%s:%d: in resource %s:\n\tmissing section 'on %s { ... }'.\n",
+			res->config_file, res->start_line, res->name,
+			nodeinfo.nodename);
+		config_valid = 0;
+	}
 	// need to verify that in the discard-node-nodename options only known
 	// nodenames are mentioned.
 	if ((opt = find_opt(res->net_options, "after-sb-0pri"))) {
@@ -2587,8 +2601,6 @@ void validate_resource(struct d_resource *res)
 			}
 		}
 	}
-	/* IP verification (check for existence)
-	 * moved to just before command execution */
 
 	if ((opt = find_opt(res->handlers, "fence-peer"))) {
 		if (strstr(opt->value, "drbd-peer-outdater"))
@@ -2711,7 +2723,7 @@ char *canonify_path(char *path)
 
 void assign_command_names_from_argv0(char **argv)
 {
-	/* in case drbdadm is called with an absolut or relative pathname
+	/* in case drbdadm is called with an absolute or relative pathname
 	 * look for the drbdsetup binary in the same location,
 	 * otherwise, just let execvp sort it out... */
 	if ((progname = strrchr(argv[0], '/')) == 0) {
@@ -2867,18 +2879,6 @@ int parse_options(int argc, char **argv)
 	return 0;
 }
 
-/* store pass through options to drbdsetup / drbdmeta etc. */
-void store_pass_through_options(int argc, char **argv)
-{
-	while (optind < argc) {
-		if (argv[optind][0] == '-' ||
-		    argv[optind][0] == ':' || isdigit(argv[optind][0]))
-			setup_opts[soi++] = argv[optind++];
-		else
-			break;
-	}
-}
-
 static void substitute_deprecated_cmd(char **c, char *deprecated,
 				      char *substitution)
 {
@@ -3045,7 +3045,7 @@ int main(int argc, char **argv)
 {
 	size_t i;
 	int rv = 0;
-	struct adm_cmd *cmd;
+	struct adm_cmd *cmd = NULL;
 	struct d_resource *res, *tmp;
 	char *env_drbd_nodename = NULL;
 	int is_dump_xml;
@@ -3082,11 +3082,18 @@ int main(int argc, char **argv)
 	rv = parse_options(argc, argv);
 	if (rv)
 		return rv;
-	store_pass_through_options(argc, argv);
+
+	/* store everything before the command name as pass through option/argument */
+	while (optind < argc) {
+		cmd = find_cmd(argv[optind]);
+		if (cmd)
+			break;
+		setup_opts[soi++] = argv[optind++];
+	}
+
 	if (optind == argc)
 		print_usage_and_exit(0);
 
-	cmd = find_cmd(argv[optind]);
 	if (cmd == NULL) {
 		fprintf(stderr, "Unknown command '%s'.\n", argv[optind]);
 		exit(E_usage);
@@ -3155,8 +3162,8 @@ int main(int argc, char **argv)
 		global_validate_maybe_expand_die_if_invalid(!is_dump);
 
 		if (optind == argc || !strcmp(argv[optind], "all")) {
-			/* either no resorce arguments at all,
-			 * but command is dump / dump-xml, so implitict "all",
+			/* either no resource arguments at all,
+			 * but command is dump / dump-xml, so implicit "all",
 			 * or an explicit "all" argument is given */
 			all_resources = 1;
 			if (!is_dump || !force)
@@ -3190,7 +3197,7 @@ int main(int argc, char **argv)
 				printf("</config>\n");
 			}
 		} else {
-			/* explicit list of reources to work on */
+			/* explicit list of resources to work on */
 			for (i = optind; (int)i < argc; i++) {
 				res = res_by_name(argv[i]);
 				if (!res)
@@ -3237,7 +3244,7 @@ int main(int argc, char **argv)
 	}
 
 	/* do we really have to bitor the exit code?
-	 * it is even only a boolen value in this case! */
+	 * it is even only a Boolean value in this case! */
 	rv |= run_dcmds();
 
 	free_config(config);
