@@ -276,7 +276,7 @@ static void pperror(struct d_host_info *host, struct d_proxy_info *proxy, char *
  * for check_uniq: check uniqueness of
  * resource names, ip:port, node:disk and node:device combinations
  * as well as resource:section ...
- * hash table to test for uniqness of these values...
+ * hash table to test for uniqueness of these values...
  *  256  (max minors)
  *  *(
  *       2 (host sections) * 4 (res ip:port node:disk node:device)
@@ -284,7 +284,7 @@ static void pperror(struct d_host_info *host, struct d_proxy_info *proxy, char *
  *     + some more,
  *       if we want to check for scoped uniqueness of *every* option
  *   )
- *     since nobody (?) will actually use more than a dozend minors,
+ *     since nobody (?) will actually use more than a dozen minors,
  *     this should be more than enough.
  */
 struct hsearch_data global_htable;
@@ -620,6 +620,7 @@ static void __parse_address(char** addr, char** port, char** af)
 		EXP(TK_IPADDR);
 		break;
 	case TK_SSOCKS:
+	case TK_SDP:
 	case TK_IPV4:
 		if (af)
 			*af = yylval.txt;
@@ -640,7 +641,7 @@ static void __parse_address(char** addr, char** port, char** af)
 		*af = strdup("ipv6");
 		break; */
 	default:
-		pe_expected("ssocks | ipv4 | ipv6 | <ipv4 address> ");
+		pe_expected("ssocks | sdp | ipv4 | ipv6 | <ipv4 address> ");
 	}
 
 	if (addr)
@@ -778,7 +779,7 @@ static void check_minor_nonsense(const char *devname, const int explicit_minor)
 	return;
 }
 
-static void parse_device(struct d_name* on_hosts, int *minor, char **device)
+static void parse_device(struct d_name* on_hosts, unsigned *minor, char **device)
 {
 	struct d_name *h;
 	int m;
@@ -808,7 +809,7 @@ static void parse_device(struct d_name* on_hosts, int *minor, char **device)
 			m = dt_minor_of_dev(*device);
 			if (m < 0) {
 				fprintf(stderr,
-					"%s:%d: no minor given nor device name conains a minor number\n",
+					"%s:%d: no minor given nor device name contains a minor number\n",
 					config_file, fline);
 				config_valid = 0;
 			}
@@ -956,7 +957,7 @@ static void parse_host_section(struct d_resource *res,
 		host->device = strdup(res->device);
 	}
 
-	if (host->device_minor == -1 && res->device_minor != -1) {
+	if (host->device_minor == -1U && res->device_minor != -1U) {
 		host->device_minor = res->device_minor;
 		for_each_host(h, on_hosts)
 			check_uniq("device-minor", "device-minor:%s:%d", h->name, host->device_minor);
@@ -970,7 +971,7 @@ static void parse_host_section(struct d_resource *res,
 
 	if (!(flags & REQUIRE_ALL))
 		return;
-	if (!host->device && host->device_minor == -1)
+	if (!host->device && host->device_minor == -1U)
 		derror(host, res, "device");
 	if (!host->disk)
 		derror(host, res, "disk");
@@ -1071,13 +1072,13 @@ static void parse_stacked_section(struct d_resource* res)
 			check_uniq("device", "device:%s:%s", h->name, host->device);
 	}
 
-	if (host->device_minor == -1 && res->device_minor != -1) {
+	if (host->device_minor == -1U && res->device_minor != -1U) {
 		host->device_minor = res->device_minor;
 		for_each_host(h, host->on_hosts)
 			check_uniq("device-minor", "device-minor:%s:%d", h->name, host->device_minor);
 	}
 
-	if (!host->device && host->device_minor == -1)
+	if (!host->device && host->device_minor == -1U)
 		derror(host, res, "device");
 	if (!host->address)
 		derror(host,res,"address");
@@ -1112,7 +1113,7 @@ void set_me_in_resource(struct d_resource* res, int match_on_proxy)
 {
 	struct d_host_info *host;
 
-	/* Determin the local host section */
+	/* Determine the local host section */
 	for (host = res->all_hosts; host; host=host->next) {
 		/* do we match  this host? */
 		if (match_on_proxy) {
@@ -1175,7 +1176,7 @@ void set_peer_in_resource(struct d_resource* res, int peer_required)
 	if (!res->me) {
 		/* should have been implicitly ignored. */
 		fprintf(stderr, "%s:%d: in resource %s:\n"
-				"\tcannot determin the peer, don't even know myself!\n",
+				"\tcannot determine the peer, don't even know myself!\n",
 				res->config_file, res->start_line, res->name);
 		exit(E_thinko);
 	}
