@@ -71,7 +71,7 @@ struct option metaopt[] = {
     { NULL,     0,              0, 0 },
 };
 
-/* FIXME? should use sector_t and off_t, not long/u64 ... */
+/* FIXME? should use sector_t and off_t, not long/uint64_t ... */
 
 /* Note RETURN VALUES:
  * exit code convention: int vXY_something() and meta_blah return some negative
@@ -124,7 +124,7 @@ struct option metaopt[] = {
  * that serves as md storage.
  *
  * For certain content on the lower level device, we should refuse
- * allways.  e.g. refuse to be created on top of a LVM2 physical volume,
+ * always.  e.g. refuse to be created on top of a LVM2 physical volume,
  * or on top of swap space. This would require people to do an dd
  * if=/dev/zero of=device.  Protects them from shooting themselves,
  * and blaming us...
@@ -148,9 +148,9 @@ struct option metaopt[] = {
 #define MD_AL_OFFSET_07        8
 #define MD_AL_MAX_SECT_07     64
 #define MD_BM_OFFSET_07        (MD_AL_OFFSET_07 + MD_AL_MAX_SECT_07)
-#define MD_RESERVED_SECT_07    ( (u64)(128ULL << 11) )
-#define MD_BM_MAX_BYTE_07      ( (u64)(MD_RESERVED_SECT_07 - MD_BM_OFFSET_07)*512 )
-#define MD_BM_MAX_BYTE_FLEX    ( (u64)(1ULL << (32-3)) )
+#define MD_RESERVED_SECT_07    ( (uint64_t)(128ULL << 11) )
+#define MD_BM_MAX_BYTE_07      ( (uint64_t)(MD_RESERVED_SECT_07 - MD_BM_OFFSET_07)*512 )
+#define MD_BM_MAX_BYTE_FLEX    ( (uint64_t)(1ULL << (32-3)) )
 
 #define DEFAULT_BM_BLOCK_SIZE  (1<<12)
 
@@ -164,7 +164,7 @@ struct option metaopt[] = {
  */
 
 /*
- * global vaiables and data types
+ * global variables and data types
  */
 
 const size_t buffer_size = 128*1024;
@@ -183,11 +183,11 @@ enum Known_Formats {
 
 /* let gcc help us get it right.
  * some explicit endian types */
-typedef struct { u64 le; } le_u64;
-typedef struct { u64 be; } be_u64;
-typedef struct { u32 le; } le_u32;
-typedef struct { u32 be; } be_u32;
-typedef struct { s32 be; } be_s32;
+typedef struct { uint64_t le; } le_u64;
+typedef struct { uint64_t be; } be_u64;
+typedef struct { uint32_t le; } le_u32;
+typedef struct { uint32_t be; } be_u32;
+typedef struct { int32_t be; } be_s32;
 typedef struct { unsigned long le; } le_ulong;
 typedef struct { unsigned long be; } be_ulong;
 
@@ -196,21 +196,21 @@ typedef struct { unsigned long be; } be_ulong;
  */
 struct md_cpu {
 	/* present since drbd 0.6 */
-	u32 gc[GEN_CNT_SIZE];	/* generation counter */
-	u32 magic;
+	uint32_t gc[GEN_CNT_SIZE];	/* generation counter */
+	uint32_t magic;
 	/* added in drbd 0.7;
 	 * 0.7 stores la_size on disk as kb, 0.8 in units of sectors.
 	 * we use sectors in our general working structure here */
-	u64 la_sect;		/* last agreed size. */
-	u32 md_size_sect;
-	s32 al_offset;		/* signed sector offset to this block */
-	u32 al_nr_extents;	/* important for restoring the AL */
-	s32 bm_offset;		/* signed sector offset to the bitmap, from here */
+	uint64_t la_sect;		/* last agreed size. */
+	uint32_t md_size_sect;
+	int32_t al_offset;		/* signed sector offset to this block */
+	uint32_t al_nr_extents;	/* important for restoring the AL */
+	int32_t bm_offset;		/* signed sector offset to the bitmap, from here */
 	/* Since DRBD 0.8 we have uuid instead of gc */
-	u64 uuid[UI_SIZE];
-	u32 flags;
-	u64 device_uuid;
-	u32 bm_bytes_per_bit;
+	uint64_t uuid[UI_SIZE];
+	uint32_t flags;
+	uint64_t device_uuid;
+	uint32_t bm_bytes_per_bit;
 };
 
 /*
@@ -239,12 +239,12 @@ struct format {
 	struct md_cpu md;
 
 	/* _byte_ offsets of our "super block" and other data, within fd */
-	u64 md_offset;
-	u64 al_offset;
-	u64 bm_offset;
+	uint64_t md_offset;
+	uint64_t al_offset;
+	uint64_t bm_offset;
 
 	/* convenience */
-	u64 bd_size; /* size of block device for internal meta data */
+	uint64_t bd_size; /* size of block device for internal meta data */
 };
 
 /* - parse is expected to exit() if it does not work out.
@@ -353,9 +353,9 @@ void md_cpu_to_disk_07(struct md_on_disk_07 *disk, const struct md_cpu const *cp
 }
 
 int is_valid_md(int f,
-	const struct md_cpu const *md, const int md_index, const u64 ll_size)
+	const struct md_cpu const *md, const int md_index, const uint64_t ll_size)
 {
-	u64 md_size_sect;
+	uint64_t md_size_sect;
 	char *v = (f == Drbd_07) ? "v07" : "v08";
 	const unsigned int magic = (f == Drbd_07) ? DRBD_MD_MAGIC_07 : DRBD_MD_MAGIC_08;
 
@@ -398,9 +398,9 @@ int is_valid_md(int f,
 		 * and the activity log; unit still sectors */
 		md_size_sect += MD_BM_OFFSET_07;
 
-		if (md->bm_offset != -(s64)md_size_sect + MD_AL_OFFSET_07) {
+		if (md->bm_offset != -(int64_t)md_size_sect + MD_AL_OFFSET_07) {
 			fprintf(stderr, "strange bm_offset %d (expected: "D64")\n",
-					md->bm_offset, -(s64)md_size_sect + MD_AL_OFFSET_07);
+					md->bm_offset, -(int64_t)md_size_sect + MD_AL_OFFSET_07);
 			return 0;
 		};
 		if (md->md_size_sect != md_size_sect) {
@@ -413,7 +413,7 @@ int is_valid_md(int f,
 		break;
 	}
 
-	/* fixme consistency check, la_size < ll_device_size,
+	/* FIXME consistency check, la_size < ll_device_size,
 	 * no overlap with internal meta data,
 	 * no overlap of flexible meta data offsets/sizes
 	 * ...
@@ -427,13 +427,13 @@ int is_valid_md(int f,
  */
 
 struct __packed al_sector_cpu {
-	u32 magic;
-	u32 tr_number;
+	uint32_t magic;
+	uint32_t tr_number;
 	struct __packed {
-		u32 pos;
-		u32 extent;
+		uint32_t pos;
+		uint32_t extent;
 	} updates[62];
-	u32 xor_sum;
+	uint32_t xor_sum;
 };
 
 struct __packed al_sector_on_disk {
@@ -717,7 +717,7 @@ void m_show_uuid(struct md_cpu *md)
 	dt_pretty_print_uuids(md->uuid,md->flags);
 }
 
-int m_strsep_u32(char **s, u32 *val)
+int m_strsep_u32(char **s, uint32_t *val)
 {
 	char *t, *e;
 	unsigned long v;
@@ -742,17 +742,17 @@ int m_strsep_u32(char **s, u32 *val)
 					*s);
 				exit(10);
 			}
-			*val = (u32)v;
+			*val = (uint32_t)v;
 		}
 		return 1;
 	}
 	return 0;
 }
 
-int m_strsep_u64(char **s, u64 *val)
+int m_strsep_u64(char **s, uint64_t *val)
 {
 	char *t, *e;
-	u64 v;
+	uint64_t v;
 
 	if ((t = strsep(s, ":"))) {
 		if (strlen(t)) {
@@ -775,9 +775,9 @@ int m_strsep_u64(char **s, u64 *val)
 	return 0;
 }
 
-int m_strsep_bit(char **s, u32 *val, int mask)
+int m_strsep_bit(char **s, uint32_t *val, int mask)
 {
-	u32 d;
+	uint32_t d;
 	int rv;
 
 	d = *val & mask ? 1 : 0;
@@ -976,7 +976,7 @@ int v06_md_initialize(struct format *cfg)
 
 void re_initialize_md_offsets(struct format *cfg)
 {
-	u64 md_size_sect;
+	uint64_t md_size_sect;
 	switch(cfg->md_index) {
 	default:
 		cfg->md.md_size_sect = MD_RESERVED_SECT_07;
@@ -1038,7 +1038,7 @@ int md_initialize_common(struct format *cfg, int do_disk_writes)
 	if (!do_disk_writes)
 		return 0;
 
-	/* do you want to initilize al to something more usefull? */
+	/* do you want to initialize al to something more useful? */
 	printf("initializing activity log\n");
 	if (MD_AL_MAX_SECT_07*512 > buffer_size) {
 		fprintf(stderr, "%s:%u: LOGIC BUG\n" , __FILE__ , __LINE__ );
@@ -1056,7 +1056,7 @@ int md_initialize_common(struct format *cfg, int do_disk_writes)
 		 * Note that even though ALIGN does round up, for sector sizes
 		 * of 512, 1024, 2048, 4096 Bytes, this will be fully within
 		 * the claimed meta data area, since we already align all
-		 * "interessting" parts of that to 4kB */
+		 * "interesting" parts of that to 4kB */
 		const size_t bm_bytes = ALIGN(cfg->bm_bytes, cfg->md_hard_sect_size);
 		size_t i = bm_bytes;
 		off_t bm_on_disk_off = cfg->bm_offset;
@@ -1091,9 +1091,9 @@ int md_initialize_common(struct format *cfg, int do_disk_writes)
  begin of v07 {{{
  ******************************************/
 
-u64 v07_style_md_get_byte_offset(const int idx, const u64 bd_size)
+uint64_t v07_style_md_get_byte_offset(const int idx, const uint64_t bd_size)
 {
-	u64 offset;
+	uint64_t offset;
 
 	switch(idx) {
 	default: /* external, some index */
@@ -1116,7 +1116,7 @@ u64 v07_style_md_get_byte_offset(const int idx, const u64 bd_size)
 	return offset;
 }
 
-unsigned long bm_words(u64 sectors, int bytes_per_bit)
+unsigned long bm_words(uint64_t sectors, int bytes_per_bit)
 {
 	unsigned long long bits;
 	unsigned long long words;
@@ -1491,8 +1491,10 @@ int meta_dstate(struct format *cfg, char **argv __attribute((unused)), int argc)
 		fprintf(stderr, "Ignoring additional arguments\n");
 	}
 
-	if (cfg->ops->open(cfg))
+	if (cfg->ops->open(cfg)) {
+		fprintf(stderr, "No valid meta data found\n");
 		return -1;
+	}
 
 	if(cfg->md.flags & MDF_CONSISTENT) {
 		if(cfg->md.flags & MDF_WAS_UP_TO_DATE) {
@@ -1532,7 +1534,7 @@ int meta_set_gi(struct format *cfg, char **argv, int argc)
 	cfg->ops->get_gi(&tmp);
 
 	if (!confirmed("Write new GI to disk?")) {
-		printf("Operation cancelled.\n");
+		printf("Operation canceled.\n");
 		exit(0);
 	}
 
@@ -1754,7 +1756,7 @@ int verify_dumpfile_or_restore(struct format *cfg, char **argv, int argc, int pa
 			cfg->md.uuid[i] = yylval.u64;
 		}
 		EXP(TK_FLAGS); EXP(TK_U32); EXP(';');
-		cfg->md.flags = (u32)yylval.u64;
+		cfg->md.flags = (uint32_t)yylval.u64;
 		EXP('}');
 	}
 	EXP(TK_LA_SIZE); EXP(TK_NUM); EXP(';');
@@ -1894,12 +1896,12 @@ void md_convert_07_to_08(struct format *cfg)
 	cfg->md.flags = cfg->md.gc[Flags];
 
 	cfg->md.uuid[UI_CURRENT] =
-		(u64)(cfg->md.gc[HumanCnt] & 0xffff) << 48 |
-		(u64)(cfg->md.gc[TimeoutCnt] & 0xffff) << 32 |
-		(u64)((cfg->md.gc[ConnectedCnt]+cfg->md.gc[ArbitraryCnt])
+		(uint64_t)(cfg->md.gc[HumanCnt] & 0xffff) << 48 |
+		(uint64_t)(cfg->md.gc[TimeoutCnt] & 0xffff) << 32 |
+		(uint64_t)((cfg->md.gc[ConnectedCnt]+cfg->md.gc[ArbitraryCnt])
 		       & 0xffff) << 16 |
-		(u64)0xbabe;
-	cfg->md.uuid[UI_BITMAP] = (u64)0;
+		(uint64_t)0xbabe;
+	cfg->md.uuid[UI_BITMAP] = (uint64_t)0;
 
 	for (i = cfg->bits_set ? UI_BITMAP : UI_HISTORY_START, j = 1;
 		i <= UI_HISTORY_END ; i++, j++)
@@ -1975,13 +1977,13 @@ struct fstype_s {
 int may_be_extX(const char *data, struct fstype_s *f)
 {
 	unsigned int size;
-	if (le16_to_cpu(*(u16*)(data+0x438)) == 0xEF53) {
+	if (le16_to_cpu(*(uint16_t*)(data+0x438)) == 0xEF53) {
 		if ( (le32_to_cpu(*(data+0x45c)) & 4) == 4 )
 			f->type = "ext3 filesystem";
 		else
 			f->type = "ext2 filesystem";
-		f->bnum  = le32_to_cpu(*(u32*)(data+0x404));
-		size     = le32_to_cpu(*(u32*)(data+0x418));
+		f->bnum  = le32_to_cpu(*(uint32_t*)(data+0x404));
+		size     = le32_to_cpu(*(uint32_t*)(data+0x418));
 		f->bsize = size == 0 ? 1024 :
 			size == 1 ? 2048 :
 			size == 2 ? 4096 :
@@ -1993,10 +1995,10 @@ int may_be_extX(const char *data, struct fstype_s *f)
 
 int may_be_xfs(const char *data, struct fstype_s *f)
 {
-	if (be32_to_cpu(*(u32*)(data+0)) == 0x58465342) {
+	if (be32_to_cpu(*(uint32_t*)(data+0)) == 0x58465342) {
 		f->type = "xfs filesystem";
-		f->bsize = be32_to_cpu(*(u32*)(data+4));
-		f->bnum  = be64_to_cpu(*(u64*)(data+8));
+		f->bsize = be32_to_cpu(*(uint32_t*)(data+4));
+		f->bnum  = be64_to_cpu(*(uint64_t*)(data+8));
 		return 1;
 	}
 	return 0;
@@ -2007,8 +2009,8 @@ int may_be_reiserfs(const char *data, struct fstype_s *f)
 	if (strncmp("ReIsErFs",data+0x10034,8) == 0 ||
 	    strncmp("ReIsEr2Fs",data+0x10034,9) == 0) {
 		f->type = "reiser filesystem";
-		f->bnum  = le32_to_cpu(*(u32*)(data+0x10000));
-		f->bsize = le16_to_cpu(*(u16*)(data+0x1002c));
+		f->bnum  = le32_to_cpu(*(uint32_t*)(data+0x10000));
+		f->bsize = le16_to_cpu(*(uint16_t*)(data+0x1002c));
 		return 1;
 	}
 	return 0;
@@ -2018,8 +2020,8 @@ int may_be_jfs(const char *data, struct fstype_s *f)
 {
 	if (strncmp("JFS1",data+0x8000,4) == 0) {
 		f->type = "JFS filesystem";
-		f->bnum = le64_to_cpu(*(u64*)(data+0x8008));
-		f->bsize = le32_to_cpu(*(u32*)(data+0x8018));
+		f->bnum = le64_to_cpu(*(uint64_t*)(data+0x8008));
+		f->bsize = le32_to_cpu(*(uint32_t*)(data+0x8018));
 		return 1;
 	}
 	return 0;
@@ -2027,8 +2029,10 @@ int may_be_jfs(const char *data, struct fstype_s *f)
 
 /* really large block size,
  * will always refuse */
-#define REFUSE_BSIZE 0xFFFFffffFFFF0000LLU
+#define REFUSE_BSIZE	0xFFFFffffFFFF0000LLU
+#define ERR_BSIZE	0xFFFFffffFFFF0001LLU
 #define REFUSE_IT()	do { f->bnum = 1; f->bsize = REFUSE_BSIZE; } while(0)
+#define REFUSE_IT_ERR()	do { f->bnum = 1; f->bsize = ERR_BSIZE; } while(0)
 int may_be_swap(const char *data, struct fstype_s *f)
 {
 	int looks_like_swap =
@@ -2044,51 +2048,109 @@ int may_be_swap(const char *data, struct fstype_s *f)
 	return 0;
 }
 
+#define N_ERR_LINES 4
+#define MAX_ERR_LINE_LEN 1024
 int guessed_size_from_pvs(struct fstype_s *f, char *dev_name)
 {
-	char buf[200];
+	char buf_in[200];
+	char *buf_err[N_ERR_LINES];
 	size_t c;
 	unsigned long long bnum;
-	int pipes[2];
+	int pipes[3][2];
+	int err_lines = 0;
+	FILE *child_err = NULL;
+	int i;
+	int ret = 0;
 	pid_t pid;
 
-	if (pipe(pipes))
+	buf_err[0] = calloc(N_ERR_LINES, MAX_ERR_LINE_LEN);
+	if (!buf_err[0])
 		return 0;
+	for (i = 1; i < N_ERR_LINES; i++)
+		buf_err[i] = buf_err[i-1] + MAX_ERR_LINE_LEN;
+
+	for (i = 0; i < 3; i++) {
+		if (pipe(pipes[i]))
+			goto out;
+	}
 
 	pid = fork();
 	if (pid < 0)
-		return 0;
+		goto out;
 
 	if (pid == 0) {
 		/* child */
 		char *argv[] = {
-			"pvs", "--noheadings", "--nosuffix", "--units", "s",
+			"pvs", "-vvv", "--noheadings", "--nosuffix", "--units", "s",
 			"-o", "pv_size",
 			dev_name,
 			NULL,
 		};
-		close(pipes[0]);
-		dup2(pipes[1],1);
-		close(pipes[1]);
-		close(2);
+		close(pipes[0][1]); /* close unused pipe ends */
+		close(pipes[1][0]);
+		close(pipes[2][0]);
+
+		dup2(pipes[0][0],0); /* map to expected stdin/out/err */
+		dup2(pipes[1][1],1);
+		dup2(pipes[2][1],2);
+
+		close(0); /* we do not use stdin */
 		execvp(argv[0], argv);
 		_exit(0);
 	}
 	/* parent */
-	close(pipes[1]);
-	c = read(pipes[0], buf, sizeof(buf)-1);
-	close(pipes[0]);
+	close(pipes[0][0]); /* close unused pipe ends */
+	close(pipes[1][1]);
+	close(pipes[2][1]);
 
-	if (c <= 0)
-		return 0;
+	close(pipes[0][1]); /* we do not use stdin in child */
 
-	buf[c] = 0;
-	if (1 == sscanf(buf, " %llu\n", &bnum)) {
-		f->bnum = bnum;
-		f->bsize = 512;
-		return 1;
-	} else
-		return 0;
+	/* We use blocking IO on pipes. This could deadlock,
+	 * If the child process would do something unexpected.
+	 * We do know the behaviour of pvs, though,
+	 * and expect only a few bytes on stdout,
+	 * and quite a few debug messages on stderr.
+	 *
+	 * First drain stderr, keeping the last N_ERR_LINES,
+	 * then read stdout. */
+	child_err = fdopen(pipes[2][0], "r");
+	if (child_err) {
+		char *b;
+		do {
+			err_lines = (err_lines + 1) % N_ERR_LINES;
+			b = fgets(buf_err[err_lines], MAX_ERR_LINE_LEN, child_err);
+		} while (b);
+	}
+
+	c = read(pipes[1][0], buf_in, sizeof(buf_in)-1);
+	if (c > 0) {
+		buf_in[c] = 0;
+		if (1 == sscanf(buf_in, " %llu\n", &bnum)) {
+			f->bnum = bnum;
+			f->bsize = 512;
+			ret = 1;
+		}
+	}
+	if (!ret) {
+		for (i = 0; i < N_ERR_LINES; i++) {
+			char *b = buf_err[(err_lines + i) % N_ERR_LINES];
+			if (b[0] == 0)
+				continue;
+			fprintf(stderr, "pvs stderr:%s", b);
+		}
+		fprintf(stderr, "\n");
+	}
+
+	i = 2;
+out:
+	for ( ; i >= 0; i--) {
+		close(pipes[i][0]);
+		close(pipes[i][1]);
+	}
+	if (child_err)
+		fclose(child_err);
+	free(buf_err[0]);
+	return ret;
 }
 
 int may_be_LVM(const char *data, struct fstype_s *f, char *dev_name)
@@ -2096,18 +2158,19 @@ int may_be_LVM(const char *data, struct fstype_s *f, char *dev_name)
 	if (strncmp("LVM2",data+0x218,4) == 0) {
 		f->type = "LVM2 physical volume signature";
 		if (!guessed_size_from_pvs(f, dev_name))
-			REFUSE_IT();
+			REFUSE_IT_ERR();
 		return 1;
 	}
 	return 0;
 }
 
+/* XXX should all this output go to stderr? */
 void check_for_existing_data(struct format *cfg)
 {
 	struct fstype_s f;
 	size_t i;
-	u64 fs_kB;
-	u64 max_usable_kB;
+	uint64_t fs_kB;
+	uint64_t max_usable_kB;
 
 	PREAD(cfg->md_fd, on_disk_buffer, SO_MUCH, 0);
 
@@ -2121,7 +2184,8 @@ void check_for_existing_data(struct format *cfg)
 	f.bnum = 0;
 	f.bsize = 0;
 
-/* FIXME add more detection magic
+/* FIXME add more detection magic.
+ * Or, rather, use some lib.
  */
 
 	(void)(
@@ -2138,14 +2202,14 @@ void check_for_existing_data(struct format *cfg)
 	 * some of the messages below only make sense for internal meta data.
 	 * for external meta data, we now only checked the meta-disk.
 	 * we should still check the actual lower level storage area for
-	 * existing data, too, and give apropriate warnings when it would
+	 * existing data, too, and give appropriate warnings when it would
 	 * appear to be truncated by too small external meta data */
 
 	printf("md_offset %llu\n", (long long unsigned)cfg->md_offset);
 	printf("al_offset %llu\n", (long long unsigned)cfg->al_offset);
 	printf("bm_offset %llu\n", (long long unsigned)cfg->bm_offset);
 
-	printf("\nFound %s ", f.type);
+	printf("\nFound %s\n", f.type);
 
 	/* FIXME overflow check missing!
 	 * relevant for ln2(bsize) + ln2(bnum) >= 64, thus only for
@@ -2160,21 +2224,16 @@ void check_for_existing_data(struct format *cfg)
 		     cfg->bm_offset )) >> 10;
 #undef min
 
-	if (ignore_sanity_checks) {
-		if (f.bsize != REFUSE_BSIZE)
-			printf("which uses "U64" kB\n", fs_kB);
-		else
-			printf("\n");
-		printf("current configuration leaves usable "U64" kB\n", max_usable_kB);
-		printf("\nIgnoring sanity check on user request.\n\n");
-		return;
-	}
 
 	if (f.bnum) {
 		if (cfg->md_index >= 0 ||
 		    cfg->md_index == DRBD_MD_INDEX_FLEX_EXT) {
+			printf("\nThis would corrupt existing data.\n");
+			if (ignore_sanity_checks) {
+				printf("\nIgnoring sanity check on user request.\n\n");
+				return;
+			}
 			printf(
-"\nThis would corrupt existing data.\n"
 "If you want me to do this, you need to zero out the first part\n"
 "of the device (destroy the content).\n"
 "You should be very sure that you mean it.\n"
@@ -2182,11 +2241,23 @@ void check_for_existing_data(struct format *cfg)
 			exit(40); /* FIXME sane exit code! */
 		}
 
-		if (f.bsize == REFUSE_BSIZE) {
+		if (f.bsize < REFUSE_BSIZE)
+			printf("%12llu kB data area apparently used\n", (unsigned long long)fs_kB);
+		printf("%12llu kB left usable by current configuration\n", (unsigned long long)max_usable_kB);
+
+		if (f.bsize == ERR_BSIZE)
 			printf(
-"\nDevice size would be truncated, which\n"
+"Could not determine the size of the actually used data area.\n\n");
+		if (f.bsize >= REFUSE_BSIZE) {
+			printf(
+"Device size would be truncated, which\n"
 "would corrupt data and result in\n"
-"'access beyond end of device' errors.\n"
+"'access beyond end of device' errors.\n");
+			if (ignore_sanity_checks) {
+				printf("\nIgnoring sanity check on user request.\n\n");
+				return;
+			}
+			printf(
 "If you want me to do this, you need to zero out the first part\n"
 "of the device (destroy the content).\n"
 "You should be very sure that you mean it.\n"
@@ -2195,8 +2266,6 @@ void check_for_existing_data(struct format *cfg)
 		}
 
 		/* looks like file system data */
-		printf("which uses "U64" kB\n", fs_kB);
-		printf("current configuration leaves usable "U64" kB\n", max_usable_kB);
 		if (fs_kB > max_usable_kB) {
 			printf(
 "\nDevice size would be truncated, which\n"
@@ -2217,7 +2286,7 @@ void check_for_existing_data(struct format *cfg)
 		printf("\n ==> This might destroy existing data! <==\n");
 
 	if (!confirmed("Do you want to proceed?")) {
-		printf("Operation cancelled.\n");
+		printf("Operation canceled.\n");
 		exit(1); // 1 to avoid online resource counting
 	}
 }
@@ -2275,17 +2344,17 @@ void check_internal_md_flavours(struct format * cfg) {
 		cfg->md_index == DRBD_MD_INDEX_FLEX_INT ? "flexible-size" : "fixed-size");
 
 	if (have_fixed_v07) {
-		fprintf(stderr, "There apears to be a v07 fixed-size internal meta data block\n"
+		fprintf(stderr, "There appears to be a v07 fixed-size internal meta data block\n"
 				"already in place on %s at byte offset %llu\n",
 				cfg->md_device_name, (long long unsigned)fixed_offset);
 	}
 	if (have_flex_v07) {
-		fprintf(stderr, "There apears to be a v07(plus) flexible-size internal meta data block\n"
+		fprintf(stderr, "There appears to be a v07(plus) flexible-size internal meta data block\n"
 				"already in place on %s at byte offset %llu",
 		cfg->md_device_name, (long long unsigned)flex_offset);
 	}
 	if (have_flex_v08) {
-		fprintf(stderr, "There apears to be a v08 flexible-size internal meta data block\n"
+		fprintf(stderr, "There appears to be a v08 flexible-size internal meta data block\n"
 				"already in place on %s at byte offset %llu",
 		cfg->md_device_name, (long long unsigned)flex_offset);
 	}
@@ -2323,7 +2392,7 @@ void check_internal_md_flavours(struct format * cfg) {
 				printf("Operation cancelled.\n");
 				exit(1); // 1 to avoid online resource counting
 			}
-			/* no need to wipe the requested flavour,
+			/* no need to wipe the requested flavor,
 			 * will be overwritten with new data */
 			cfg->md.magic = 0;
 			if (cfg->md_index == DRBD_MD_INDEX_INTERNAL)
@@ -2430,7 +2499,7 @@ int meta_create_md(struct format *cfg, char **argv __attribute((unused)), int ar
 	 * on a request to create v08 flex-internal meta data (or v07 plus, for
 	 * that matter), we also check the same offset for the respective other
 	 * flex-internal format version, as well as the v07 fixed-size internal
-	 * meta data offset for its flavour of meta data.
+	 * meta data offset for its flavor of meta data.
 	 */
 	if (cfg->md_index == DRBD_MD_INDEX_INTERNAL ||
 	    cfg->md_index == DRBD_MD_INDEX_FLEX_INT)
@@ -2468,7 +2537,7 @@ int meta_wipe_md(struct format *cfg, char **argv __attribute((unused)), int argc
 
 	virgin = cfg->ops->open(cfg);
 	if (virgin) {
-		fprintf(stderr,"There apears to be no drbd meta data to wipe out?\n");
+		fprintf(stderr,"There appears to be no drbd meta data to wipe out?\n");
 		return 0;
 	}
 
@@ -2632,7 +2701,7 @@ int parse_format(struct format *cfg, char **argv, int argc, int *ai)
 int is_attached(int minor)
 {
 	FILE *pr;
-	char token[128];	/* longest interessting token is 40 Byte (git hash) */
+	char token[128];	/* longest interesting token is 40 Byte (git hash) */
 	int rv = -1;
 	long m, cm = -1;
 	char *p;
@@ -2784,7 +2853,7 @@ int main(int argc, char **argv)
 	}
 	ai++;
 
-	/* does exit() unless we aquired the lock.
+	/* does exit() unless we acquired the lock.
 	 * unlock happens implicitly when the process dies,
 	 * but may be requested implicitly
 	 */
