@@ -46,6 +46,7 @@
 #include <time.h>
 
 #define __bitwise /* Build-workaround for broken RHEL4 kernels (2.6.9_78.0.1) */
+#include <linux/types.h>
 #include <linux/netlink.h>
 #include <linux/connector.h>
 
@@ -60,6 +61,11 @@
 #error "You need to set KDIR while building drbdsetup."
 #endif
 
+#ifndef AF_INET_SDP
+#define AF_INET_SDP 27
+#define PF_INET_SDP AF_INET_SDP
+#endif
+
 enum usage_type {
 	BRIEF,
 	FULL,
@@ -112,7 +118,7 @@ struct drbd_option {
 struct drbd_cmd {
 	const char* cmd;
 	const int packet_id;
-	int (*function)(struct drbd_cmd *, int, int, char **);
+	int (*function)(struct drbd_cmd *, unsigned, int, char **);
 	void (*usage)(struct drbd_cmd *, enum usage_type);
 	union {
 		struct {
@@ -120,7 +126,7 @@ struct drbd_cmd {
 			struct drbd_option *options;
 		} cp; // for generic_config_cmd, config_usage
 		struct {
-			int (*show_function)(struct drbd_cmd *, int,
+			int (*show_function)(struct drbd_cmd *, unsigned,
 					     unsigned short* );
 		} gp; // for generic_get_cmd, get_usage
 		struct {
@@ -146,10 +152,10 @@ static int get_af_ssocks(int warn);
 static void print_command_usage(int i, const char *addinfo, enum usage_type);
 
 // command functions
-static int generic_config_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv);
-static int down_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv);
-static int generic_get_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv);
-static int events_cmd(struct drbd_cmd *cm, int minor, int argc,char **argv);
+static int generic_config_cmd(struct drbd_cmd *cm, unsigned minor, int argc, char **argv);
+static int down_cmd(struct drbd_cmd *cm, unsigned minor, int argc, char **argv);
+static int generic_get_cmd(struct drbd_cmd *cm, unsigned minor, int argc, char **argv);
+static int events_cmd(struct drbd_cmd *cm, unsigned minor, int argc,char **argv);
 
 // usage functions
 static void config_usage(struct drbd_cmd *cm, enum usage_type);
@@ -169,13 +175,13 @@ static void bit_opt_xml(struct drbd_option *option);
 static void string_opt_xml(struct drbd_option *option);
 
 // sub commands for generic_get_cmd
-static int show_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
-static int role_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
-static int status_xml_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
-static int sh_status_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
-static int cstate_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
-static int dstate_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
-static int uuids_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl);
+static int show_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
+static int role_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
+static int status_xml_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
+static int sh_status_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
+static int cstate_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
+static int dstate_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
+static int uuids_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl);
 
 // convert functions for arguments
 static int conv_block_dev(struct drbd_argument *ad, struct drbd_tag_list *tl, char* arg);
@@ -449,6 +455,7 @@ static const char *error_messages[] = {
 	EM(ERR_CSUMS_ALG) = "CSUMSAlgNotAvail",
 	EM(ERR_CSUMS_ALG_ND) = "CSUMSAlgNotDigest",
 	EM(ERR_CSUMS_RESYNC_RUNNING) = "Can not change csums-alg while resync is in progress",
+	EM(ERR_PERM) = "Permission denied. CAP_SYS_ADMIN necessary",
 };
 #define MAX_ERROR (sizeof(error_messages)/sizeof(*error_messages))
 const char * error_to_string(int err_no)
@@ -474,7 +481,7 @@ static int dump_tag_list(unsigned short *tlc)
 	int len;
 	int integer;
 	char bit;
-	__u64 int64;
+	uint64_t int64;
 	const char* string;
 	int found_unknown=0;
 
@@ -496,7 +503,7 @@ static int dump_tag_list(unsigned short *tlc)
 			printf("(integer) %d",integer);
 			break;
 		case TT_INT64:
-			int64 = *(__u64*)tlc;
+			int64 = *(uint64_t*)tlc;
 			printf("(int64) %lld",(long long)int64);
 			break;
 		case TT_BIT:
@@ -643,6 +650,7 @@ static void split_address(char* text, int *af, char** address, int* port)
 	static struct { char* text; int af; } afs[] = {
 		{ "ipv4:", AF_INET  },
 		{ "ipv6:", AF_INET6 },
+		{ "sdp:",  AF_INET_SDP },
 		{ "ssocks:",  -1 },
 	};
 
@@ -737,8 +745,10 @@ static int conv_bit(struct drbd_option *od, struct drbd_tag_list *tl, char* arg
 
 /* It will only print the WARNING if the warn flag is set
    with the _first_ call! */
-#define PROC_NET_AF_SSOCKS_FAMILY "/proc/net/af_sci/family"
-static int get_af_ssocks(int warn)
+#define PROC_NET_AF_SCI_FAMILY "/proc/net/af_sci/family"
+#define PROC_NET_AF_SSOCKS_FAMILY "/proc/net/af_ssocks/family"
+
+static int get_af_ssocks(int warn_and_use_default)
 {
 	char buf[16];
 	int c, fd;
@@ -748,12 +758,17 @@ static int get_af_ssocks(int warn)
 		return af;
 
 	fd = open(PROC_NET_AF_SSOCKS_FAMILY, O_RDONLY);
+
+	if (fd < 0)
+		fd = open(PROC_NET_AF_SCI_FAMILY, O_RDONLY);
+
 	if (fd < 0) {
-		if (warn)
+		if (warn_and_use_default) {
 			fprintf(stderr, "open(" PROC_NET_AF_SSOCKS_FAMILY ") "
 				"failed: %m\n WARNING: assuming AF_SSOCKS = 27. "
-				"Socket creation will probabely fail.\n");
-		af = 27;
+				"Socket creation may fail.\n");
+			af = 27;
+		}
 		return af;
 	}
 	c = read(fd, buf, sizeof(buf)-1);
@@ -763,11 +778,12 @@ static int get_af_ssocks(int warn)
 			buf[c-1] = 0;
 		af = m_strtoll(buf,1);
 	} else {
-		if (warn)
+		if (warn_and_use_default) {
 			fprintf(stderr, "read(" PROC_NET_AF_SSOCKS_FAMILY ") "
 				"failed: %m\n WARNING: assuming AF_SSOCKS = 27. "
-				"Socket creation will probabely fail.\n");
-		af = 27;
+				"Socket creation may fail.\n");
+			af = 27;
+		}
 	}
 	close(fd);
 	return af;
@@ -934,15 +950,16 @@ static int print_config_error(int err_no)
 			// Ignore SS_SUCCESS, SS_NOTHING_TO_DO, SS_CW_Success...
 		} else {
 			fprintf(stderr,"%s: State change failed: (%d) %s\n",
-				devname, err_no, set_st_err_name(err_no));
+				devname, err_no, drbd_set_st_err_str(err_no));
 			if (err_no == SS_NO_UP_TO_DATE_DISK) {
-				/* am R_PRIMARY, cannot outdate */
+				/* all available disks are inconsistent,
+				 * or I am consistent, but cannot outdate the peer. */
 				rv = 17;
 			} else if (err_no == SS_LOWER_THAN_OUTDATED) {
 				/* was inconsistent anyways */
 				rv = 5;
 			} else if (err_no == SS_NO_LOCAL_DISK) {
-				/* Can not start resync, no local diks, try with drbdmeta */
+				/* Can not start resync, no local disks, try with drbdmeta */
 				rv = 16;
 			} else {
 				rv = 11;
@@ -994,7 +1011,7 @@ static void dump_argv(int argc, char **argv, int first_non_option, int n_known_a
 	fprintf(stderr, "`--\n");
 }
 
-static int _generic_config_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv)
+static int _generic_config_cmd(struct drbd_cmd *cm, unsigned minor, int argc, char **argv)
 {
 	char buffer[ RCV_SIZE ];
 	struct drbd_nl_cfg_reply *reply;
@@ -1084,7 +1101,7 @@ error:
 	return rv;
 }
 
-static int generic_config_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv)
+static int generic_config_cmd(struct drbd_cmd *cm, unsigned minor, int argc, char **argv)
 {
 	return print_config_error(_generic_config_cmd(cm, minor, argc, argv));
 }
@@ -1103,8 +1120,8 @@ static void show_numeric(struct drbd_option *od, unsigned short* tp)
 		val = get_unaligned((int*)tp);
 		break;
 	case TT_INT64:
-		ASSERT( get_unaligned(tp++) == sizeof(__u64) );
-		val = get_unaligned((__u64*)tp);
+		ASSERT( get_unaligned(tp++) == sizeof(uint64_t) );
+		val = get_unaligned((uint64_t*)tp);
 		break;
 	default:
 		ASSERT(0);
@@ -1265,7 +1282,7 @@ static int consume_tag_bit(enum drbd_tags tag, unsigned short *tlc, int* val)
 	return 0;
 }
 
-static int generic_get_cmd(struct drbd_cmd *cm, int minor, int argc,
+static int generic_get_cmd(struct drbd_cmd *cm, unsigned minor, int argc,
 		    char **argv __attribute((unused)))
 {
 	char buffer[ 4096 ];
@@ -1328,8 +1345,14 @@ static char *af_to_str(int af)
 		return "ipv4";
 	else if (af == AF_INET6)
 		return "ipv6";
+	/* AF_SSOCKS typically is 27, the same as AF_INET_SDP.
+	 * But with warn_and_use_default = 0, it will stay at -1 if not available.
+	 * Just keep the test on ssocks before the one on SDP (which is hard-coded),
+	 * and all should be fine.  */
 	else if (af == get_af_ssocks(0))
 		return "ssocks";
+	else if (af == AF_INET_SDP)
+		return "sdp";
 	else return "unknown";
 }
 
@@ -1341,7 +1364,9 @@ static void show_address(void* address, int addr_len)
 	char buffer[INET6_ADDRSTRLEN];
 
 	addr = (struct sockaddr *)address;
-	if (addr->sa_family == AF_INET || addr->sa_family == get_af_ssocks(0)) {
+	if (addr->sa_family == AF_INET
+	|| addr->sa_family == get_af_ssocks(0)
+	|| addr->sa_family == AF_INET_SDP) {
 		addr4 = (struct sockaddr_in *)address;
 		printf("\taddress\t\t\t%s %s:%d;\n",
 		       af_to_str(addr4->sin_family),
@@ -1358,7 +1383,7 @@ static void show_address(void* address, int addr_len)
 	}
 }
 
-static int show_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl)
+static int show_scmd(struct drbd_cmd *cm, unsigned minor, unsigned short *rtl)
 {
 	int idx = idx;
 	char *str = NULL, *backing_dev, *address;
@@ -1370,7 +1395,7 @@ static int show_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl)
 			print_options(cm->cp.options, rtl, cm->cmd);
 	}
 
-	// start of spagethi code...
+	// start of spaghetti code...
 	if(consume_tag_int(T_wire_protocol,rtl,&idx))
 		printf("protocol %c;\n",'A'+idx-1);
 	backing_dev = address = NULL;
@@ -1413,7 +1438,7 @@ static int show_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl)
 }
 
 static int status_xml_scmd(struct drbd_cmd *cm __attribute((unused)),
-		int minor, unsigned short *rtl)
+		unsigned minor, unsigned short *rtl)
 {
 	union drbd_state state = { .i = 0 };
 	int synced = 0;
@@ -1440,11 +1465,11 @@ static int status_xml_scmd(struct drbd_cmd *cm __attribute((unused)),
 		" ro1=\"%s\" ro2=\"%s\""
 		/* disk state */
 		" ds1=\"%s\" ds2=\"%s\"",
-	       conns_to_name(state.conn),
-	       roles_to_name(state.role),
-	       roles_to_name(state.peer),
-	       disks_to_name(state.disk),
-	       disks_to_name(state.pdsk));
+	       drbd_conn_str(state.conn),
+	       drbd_role_str(state.role),
+	       drbd_role_str(state.peer),
+	       drbd_disk_str(state.disk),
+	       drbd_disk_str(state.pdsk));
 
 	/* io suspended ? */
 	if (state.susp)
@@ -1465,7 +1490,7 @@ static int status_xml_scmd(struct drbd_cmd *cm __attribute((unused)),
 }
 
 static int sh_status_scmd(struct drbd_cmd *cm __attribute((unused)),
-		int minor, unsigned short *rtl)
+		unsigned minor, unsigned short *rtl)
 {
 /* variable prefix; maybe rather make that a command line parameter?
  * or use "drbd_sh_status"? */
@@ -1504,11 +1529,11 @@ static int sh_status_scmd(struct drbd_cmd *cm __attribute((unused)),
 			"%s_disk=%s\n"
 			"%s_pdsk=%s\n\n",
 			_P,
-			_P, conns_to_name(state.conn),
-			_P, roles_to_name(state.role),
-			_P, roles_to_name(state.peer),
-			_P, disks_to_name(state.disk),
-			_P, disks_to_name(state.pdsk));
+			_P, drbd_conn_str(state.conn),
+			_P, drbd_role_str(state.role),
+			_P, drbd_role_str(state.peer),
+			_P, drbd_disk_str(state.disk),
+			_P, drbd_disk_str(state.pdsk));
 
 		/* io suspended ? */
 		printf("%s_flags_susp=%s\n", _P, state.susp ? "1" : "");
@@ -1532,7 +1557,7 @@ static int sh_status_scmd(struct drbd_cmd *cm __attribute((unused)),
 }
 
 static int role_scmd(struct drbd_cmd *cm __attribute((unused)),
-	       int minor __attribute((unused)),
+	       unsigned minor __attribute((unused)),
 	       unsigned short *rtl)
 {
 	union drbd_state state = { .i = 0 };
@@ -1541,13 +1566,13 @@ static int role_scmd(struct drbd_cmd *cm __attribute((unused)),
 	     state.disk == D_DISKLESS) {
 		printf("Unconfigured\n");
 	} else {
-		printf("%s/%s\n",roles_to_name(state.role),roles_to_name(state.peer));
+		printf("%s/%s\n",drbd_role_str(state.role),drbd_role_str(state.peer));
 	}
 	return 0;
 }
 
 static int cstate_scmd(struct drbd_cmd *cm __attribute((unused)),
-		int minor __attribute((unused)),
+		unsigned minor __attribute((unused)),
 		unsigned short *rtl)
 {
 	union drbd_state state = { .i = 0 };
@@ -1556,13 +1581,13 @@ static int cstate_scmd(struct drbd_cmd *cm __attribute((unused)),
 	     state.disk == D_DISKLESS) {
 		printf("Unconfigured\n");
 	} else {
-		printf("%s\n",conns_to_name(state.conn));
+		printf("%s\n",drbd_conn_str(state.conn));
 	}
 	return 0;
 }
 
 static int dstate_scmd(struct drbd_cmd *cm __attribute((unused)),
-		int minor __attribute((unused)),
+		unsigned minor __attribute((unused)),
 		unsigned short *rtl)
 {
 	union drbd_state state = { .i = 0 };
@@ -1571,26 +1596,26 @@ static int dstate_scmd(struct drbd_cmd *cm __attribute((unused)),
 	     state.disk == D_DISKLESS) {
 		printf("Unconfigured\n");
 	} else {
-		printf("%s/%s\n",disks_to_name(state.disk),disks_to_name(state.pdsk));
+		printf("%s/%s\n",drbd_disk_str(state.disk),drbd_disk_str(state.pdsk));
 	}
 	return 0;
 }
 
 static int uuids_scmd(struct drbd_cmd *cm,
-	       int minor __attribute((unused)),
+	       unsigned minor __attribute((unused)),
 	       unsigned short *rtl)
 {
-	__u64 *uuids;
+	uint64_t *uuids;
 	int flags = flags;
 	unsigned int len;
 
 	if(!consume_tag_blob(T_uuids,rtl,(char **) &uuids,&len)) {
 		fprintf(stderr,"Reply payload did not carry an uuid-tag,\n"
-			"Probabely the device has no disk!\n");
+			"Probably the device has no disk!\n");
 		return 1;
 	}
 	consume_tag_int(T_uuids_flags,rtl,&flags);
-	if( len == UI_SIZE * sizeof(__u64)) {
+	if( len == UI_SIZE * sizeof(uint64_t)) {
 		if(!strcmp(cm->cmd,"show-gi")) {
 			dt_pretty_print_uuids(uuids,flags);
 		} else if(!strcmp(cm->cmd,"get-gi")) {
@@ -1623,7 +1648,7 @@ static struct drbd_cmd *find_cmd_by_name(char *name)
 	return NULL;
 }
 
-static int down_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv)
+static int down_cmd(struct drbd_cmd *cm, unsigned minor, int argc, char **argv)
 {
 	int rv;
 	int success;
@@ -1767,11 +1792,11 @@ static int print_broadcast_events(unsigned int seq, int u __attribute((unused)),
 			printf("%u ST %d { cs:%s ro:%s/%s ds:%s/%s %c%c%c%c }\n",
 			       seq,
 			       reply->minor,
-			       conns_to_name(state.conn),
-			       roles_to_name(state.role),
-			       roles_to_name(state.peer),
-			       disks_to_name(state.disk),
-			       disks_to_name(state.pdsk),
+			       drbd_conn_str(state.conn),
+			       drbd_role_str(state.role),
+			       drbd_role_str(state.peer),
+			       drbd_disk_str(state.disk),
+			       drbd_disk_str(state.pdsk),
 			       state.susp ? 's' : 'r',
 			       state.aftr_isp ? 'a' : '-',
 			       state.peer_isp ? 'p' : '-',
@@ -1837,7 +1862,7 @@ static int w_synced_state(unsigned int seq __attribute((unused)),
 	return 1;
 }
 
-static int events_cmd(struct drbd_cmd *cm, int minor, int argc ,char **argv)
+static int events_cmd(struct drbd_cmd *cm, unsigned minor, int argc ,char **argv)
 {
 	void *buffer;
 	struct cn_msg *cn_reply;
@@ -2011,14 +2036,14 @@ static int events_cmd(struct drbd_cmd *cm, int minor, int argc ,char **argv)
 			if (cn_reply->ack == 0) { // broadcasts
 				if (cn_reply->seq <= b_seq) continue;
 				b_seq = cn_reply->seq;
-			} else if (minor == (int)reply->minor && cn_reply->ack == (__u32)getpid() + 1) {
-				// replies to drbdsetup packes and for this device.
+			} else if (minor == reply->minor && cn_reply->ack == (uint32_t)getpid() + 1) {
+				// replies to drbdsetup packets and for this device.
 				if (cn_reply->seq <= r_seq) continue;
 				r_seq = cn_reply->seq;
 			}
 		}
 
-		if( all_devices || minor == (int)reply->minor ) {
+		if( all_devices || minor == reply->minor ) {
 			cont=cm->ep.proc_event(cn_reply->seq, wasb, reply);
 		}
 	} while(cont);
@@ -2302,7 +2327,7 @@ static int open_cn()
 
 static void prepare_nl_header(struct nlmsghdr* nl_hdr, int size)
 {
-	static __u32 cn_seq = 1;
+	static uint32_t cn_seq = 1;
 	struct cn_msg *cn_hdr;
 	cn_hdr = (struct cn_msg *)NLMSG_DATA(nl_hdr);
 
@@ -2436,12 +2461,12 @@ static int is_drbd_driver_missing(void)
 
 int main(int argc, char** argv)
 {
-	int minor;
+	unsigned minor;
 	struct drbd_cmd *cmd;
 	int rv=0;
 
 	if (chdir("/")) {
-		/* highliy unlikely, but gcc is picky */
+		/* highly unlikely, but gcc is picky */
 		perror("cannot chdir /");
 		return -111;
 	}
